MODULE ExposureDoseModule                                            ! Compile after the modules below
   ! Latest revisions by WGG on August 14, 2012
   USE GlobalModule                                                  ! Compile this module first
   USE UtilityModule                                                 ! Compile this module second
   USE DistributionModule                                            ! Compile this module third
   USE PMModule                                                      ! Compile this module fourth
   IMPLICIT NONE                                                     ! Must declare all variables
   
   REAL  (KIND=LONG), ALLOCATABLE :: PAIArray(:)                     ! Buffer for PAI data
   
CONTAINS                                                             ! Routines internal to this module

SUBROUTINE Ventilation(P)                                            ! Called from main APEX program
   INTEGER (KIND=IK), INTENT(IN) :: P                                ! Input variable (profile number)
   INTEGER (KIND=IK), PARAMETER  :: MAXEVENTS = 300000               ! Max # of events per person
   INTEGER (KIND=IK) :: I, J, K, T                                   ! Local integer variables
   REAL    (KIND=RK) :: RecoveryT                                    ! Local real variables
   REAL    (KIND=RK) :: origmets, EQUIV                              ! Local real variables
   REAL    (KIND=RK) :: METSFactor, METSMax                          ! Various limits on METS
   REAL    (KIND=RK) :: LogTerm, LogVEBM, VO2Factor                  ! More local real variables
   REAL    (KIND=RK) :: Z(MAXEVENTS), Random                         ! More local real variables
   REAL    (KIND=RK) :: Sfast, Dmax, Duration, M, Mlast              ! More local real variables
   REAL    (KIND=RK) :: Flast, DelM, EPOCfast, EPOCslow, METS, Del2  ! More local real variables
   REAL    (KIND=RK) :: DeltaMETS, DelFfast, DelFslow                ! More local real variables
   REAL    (KIND=RK) :: A, B, Fraction                               ! More local real variables
   REAL    (KIND=RK) :: Median(1)=(/50.0/), PAI(1)                   ! Array for median percentile
   ! Global constants: IK, LogU, LV_MS2, RK, SL_FM2, ST_MS2, YES
   ! Global vars used: DebugLevel, DoDose, NumDays, NumEvents, NumHours, Phys
   ! Global vars set : EventSeq, FirstProfile, HourSeq, ISdPrs, PersonDay, ProcLevel, ProcName
   ! Module vars set : PAIArray
   ! Intrinsic procs : EXP, LOG, MAX, MAXVAL, MIN, MOD, RANDOM_SEED, REAL, SUM, WRITE
   ! APEX procedures : RnNor, Fatal
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "Ventilation"                               ! Procedure name for messages
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) ST_MS2, ProcName(ProcLevel)  ! Message for debugging
   IF (P==FirstProfile) THEN                                         ! If start of simulation
      ALLOCATE(PAIArray(NumDays),STAT=AllocErr)                      ! Buffer for PAI values
      CALL CheckAllocation(AllocErr)                                 ! CheckAllocation
   ENDIF                                                             ! End start simulation logic   
   EventSeq(:)%VA  = 0.                                              ! Initialize VA value per event
   EventSeq(:)%VE  = 0.                                              ! Initialize VE value per event
   HourSeq(:)%VE   = 0.                                              ! Initialize VE value per hour
   HourSeq(:)%VA   = 0.                                              ! Initialize VA value per hour
   HourSeq(:)%EE   = 0.                                              ! Initialize EE value per hour
   CALL RANDOM_SEED(Put=ISdVent(1:KSeed))                            ! Initialize ventilation seed
   IF (NumEvents>MAXEVENTS) Call Fatal(1)                            ! Array bounds of Z exceeded
   CALL RnNor(NumEvents,Z,-4D0,4D0)                                  ! Pick NumEvents normal numbers 
   CALL RANDOM_NUMBER(Random)                                        ! Generate uniform
   RecoveryT       = Phys(P)%RecTime                                 ! Recovery time (hours)
   METSMax         = Phys(P)%MetsMax                                 ! METS limit for profile P
   METSFactor      = Phys(P)%ECF*Phys(P)%RMR*19630.                  ! METS to VA conversion factor
   VO2Factor       = Phys(P)%ECF*Phys(P)%RMR/Phys(P)%BM              ! VO2/BM at rest (l-O2/min/kg)
   PersonDay%EndGn = Phys(P)%ENDGN1                                  ! Normal EndGn rate (all days)
   K               = Phys(P)%Start                                   ! Phase of menstrual cycle
   IF (K>0) THEN                                                     ! If menstrual cycle exists
      DO J=1,NumDays                                                 ! Loop over days in simulation 
         IF (1+MOD(J+K,28)>14) PersonDay(J)%EndGn = Phys(P)%ENDGN2   ! Set 14 days per 28 to ENDGN2
      ENDDO                                                          ! Continue with next day  
   ENDIF                                                             ! End of ENDGN logic
   Flast        = 0.                                                 ! Initialize O2 debt      
   Mlast        = 0.                                                 ! Initialize prev METS value
   RecoveryT    = Phys(P)%RecTime                                    ! Recovery time (hours)
   MetsMax      = Phys(P)%MetsMax                                    ! METS limit for profile P
   A            = 5.20-(1.54/RecoveryT)+(3.92/RecoveryT**2.0)        ! O2 deficit regression coeff A
   B            = 3.93-(3.57/RecoveryT)+(3.66/RecoveryT**2.0)        ! O2 deficit regression coeff B
   EQUIV        = (Phys(P)%RMR*Phys(P)%ECF*1000)/Phys(P)%BM          ! Equivalency conversion for person
   Dmax         = Phys(P)%MOXD/(EQUIV*60.*(METSMax-1.))              ! Convert MOD to Dmax in (M-hr) 
   Sfast        = (0.6+3.1*Random)*60./(METSMax-1.)                  ! Slope of fast recovery (M/hr)   
   DO I=1,NumEvents                                                  ! Loop over events   
      Origmets  = EventSeq(I)%METS                                   ! Current unadjusted METS
      Mets      = MIN(MAX(Origmets,1.001),MetsMax)                   ! METS, bounded by 1 and METSmax
      Duration  = EventSeq(I)%Duration/60.                           ! Duration in hours
      M         = (Mets-1.)/(MetsMax-1.)                             ! Unadjusted value for M 
      DO WHILE (M>0)                                                 ! Loop until fatigue is acceptable
         DelM      = M-Mlast                                         ! Change in M from prior event  
         DelFfast  = 0.5*((DelM)*(abs(DelM)))/(Sfast*Dmax)           ! DeltaF fast   
         Del2      = ABS(DelM)-Sfast*Duration                        ! Determine if Fast is truncated
         IF (Del2 > 0.) DelFfast = DelFfast*(1.-Del2**2/DelM**2)     ! Cut DelF(fast) if truncated
         DelFslow  = (A*(M**B)-1./RecoveryT)*Duration                ! Calculate DelF(slow)
         IF (Flast+DelFfast+DelFslow < 0.) THEN                      ! If more than full recovery  
            fraction = Flast/ABS(DelFfast+DelFslow)                  ! Determine excess recovery  
            DelFFast = DelFfast*fraction                             ! Limit fast recovery
            DelFSlow = DelFslow*fraction                             ! Limit slow recovery  
         ENDIF                                                       ! End excess recovery correction 
         EventSeq(I)%Deficit = Flast + DelFfast + DelFslow           ! New deficit (F) value 
         IF (EventSeq(I)%Deficit < 1.) EXIT                          ! If F value is ok, exit
         M = M - 0.01                                                ! Adjust M downwards and try again
      ENDDO                                                          ! End fatigue adjustment logic
      EPOCfast = 0.                                                  ! Default is no fast EPOC 
      IF (DelFfast<0.) EPOCfast = ABS(DelFfast)*Dmax/duration        ! Determine fast EPOC
      EPOCslow = 0.                                                  ! Default is no slow EPOC
      IF (DelFslow<0.) EPOCslow = ABS(DelFslow)*Dmax/duration        ! Determine slow EPOC
      M = M + EPOCfast + EPOCslow                                    ! Adjust M for EPOC
      DeltaMets = 1.+M*(MetsMax-1.)-Mets                             ! Change in absolute METS
      IF (OrigMets<1.)  Mets = OrigMets                              ! Return METS<1 to orig vals   
      EventSeq(I)%Mets = Mets+DeltaMets                              ! Update event METS w/EPOC 
      if (EventSeq(I)%Deficit<0) EventSeq(I)%Deficit=0               ! Fix any roundoff error in F
      Flast = EventSeq(I)%Deficit                                    ! Update last F 
      Mlast = M                                                      ! Update last M
   ENDDO                                                             ! End loop over events
   DO I=1,NumEvents                                                  ! Loop over diary events
      EventSeq(I)%VA = EventSeq(I)%METS * METSFactor                 ! VA is proportional to METS
      LogTerm        = LOG(EventSeq(I)%METS*VO2Factor)               ! Log(VO2/BM) term
      LogVEBM        = Phys(P)%VEinter + Phys(P)%VEslope*LogTerm    & 
                      +Phys(P)%VEresid*Z(I)                          ! Regression for Log(VE/BM)    
      EventSeq(I)%VE = Phys(P)%BM * EXP(LogVEBM) * 1000.             ! Solve for VE in (ml/min)
      IF (EventSeq(I)%Duration > 5) THEN                             ! For events longer than 5 min
         EventSeq(I)%VE = MIN(EventSeq(I)%VE, 100000.)               ! Max Ve is 100 L/min   
      ELSE                                                           ! Otherwise   
         EventSeq(I)%VE = MIN(EventSeq(I)%VE, 150000.)               ! Max Ve is 150 L/min
      ENDIF                                                          ! End of max Ve logic
      EventSeq(I)%EVR = EventSeq(I)%VE/(1000*(Phys(P)%BSA))          ! EVR for event (l/min-m^2)  
   ENDDO                                                             ! Continue with next diary event
   DO I=1,NumHours                                                   ! Loop for update hour variables
      J = HourSeq(I)%FirstEvent                                      ! First event for given hour 
      K = HourSeq(I)%LastEvent                                       ! Last event for given hour      
      HourSeq(I)%Mets =                                             & 
         SUM(EventSeq(J:K)%Mets*EventSeq(J:K)%Duration)/60.          ! Mean METS value for given hour 
      HourSeq(I)%EE = Phys(P)%RMR*HourSeq(I)%Mets                    ! Energy expenditure kcal/min       
      HourSeq(I)%VE =                                               & 
         SUM(EventSeq(J:K)%VE*EventSeq(J:K)%Duration)/60.            ! Mean VE value for given hour 
      HourSeq(I)%VA =                                               & 
         SUM(EventSeq(J:K)%VA*EventSeq(J:K)%Duration)/60.            ! Mean VA value for given hour 
      HourSeq(I)%EVR = HourSeq(I)%VE/(1000*(Phys(P)%BSA))            ! Mean EVR for hour (l/min-m^2)  
      HourSeq(I)%PAI = HourSeq(I)%Mets                               ! Mean PAI value for given hour
      IF (I<=8) THEN                                                 ! First 8 hours use special logic 
         HourSeq(I)%Run8EVR  = SUM(HourSeq(1:I)%EVR)/I               ! Running 8-hour average for EVR
      ELSE                                                           ! If beyond first 8 hours
         HourSeq(I)%Run8EVR  = HourSeq(I-1)%Run8EVR  +              &
            (HourSeq(I)%EVR-HourSeq(I-8)%EVR)/8.                     ! Add new hour and drop off old
      ENDIF                                                          ! End of running 8-hour logic           
   ENDDO                                                             ! End hour sequence update           
   T =  TSMinutes                                                    ! Local copy of minutes/timestep
   DO I=1,NumTS                                                      ! Loop for update ts variables
      J = TSSeq(I)%FirstEvent                                        ! First event for given ts 
      K = TSSeq(I)%LastEvent                                         ! Last event for given ts
      TSSeq(I)%Mets =                                            & 
         SUM(EventSeq(J:K)%Mets*EventSeq(J:K)%Duration)/REAL(T)      ! Mean METS value for given ts 
      TSSeq(I)%EE = Phys(P)%RMR*TSSeq(I)%Mets                        ! Energy expenditure kcal/min       
      TSSeq(I)%VE =                                              & 
         SUM(EventSeq(J:K)%VE*EventSeq(J:K)%Duration)/REAL(T)        ! Mean VE value for given ts 
      TSSeq(I)%VA =                                              & 
         SUM(EventSeq(J:K)%VA*EventSeq(J:K)%Duration)/REAL(T)        ! Mean VA value for given ts 
      TSSeq(I)%EVR = TSSeq(I)%VE/(1000*(Phys(P)%BSA))                ! Mean EVR for ts (l/min-m^2)  
      TSSeq(I)%PAI = TSSeq(I)%Mets                                   ! Mean PAI value for given ts
   ENDDO                                                             ! End hour sequence update           
   DO I=1,NumDays                                                    ! Loop over days in simulation
      PersonDay(I)%PAI = SUM(HourSeq(24*I-23:24*I)%PAI)/24           ! Daily average PAI  
   ENDDO                                                             ! End day loop   
   PAIArray=REAL(PersonDay%PAI,LONG)                                 ! Fill the temp PAI array
   CALL Percentiles(PAIArray,NumDays,Median(1),1,PAI)                ! Get median PAI over days 
   Person(P)%PAI = PAI(1)                                            ! Assign the median PAI for person  
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE Ventilation                                           ! Return to main APEX program

SUBROUTINE Exposure(P, PL)                                           ! Called from main APEX program
   INTEGER (KIND=IK), INTENT(IN) :: P, PL                            ! Input variables (person, pollut)
   INTEGER (KIND=IK) :: D, I, J, K, L, M, NOver, TOver, T            ! Local integer variables
   INTEGER (KIND=IK) :: Location, LastLoc                            ! Local integer variables
   REAL    (KIND=RK) :: A, B, C, VE, Tot, VE_BSA, Y                  ! Local real variables
   REAL    (KIND=RK) :: ExpF = 0.0                                   ! Conversion from input to output units
   REAL    (KIND=RK) :: ExpFEV = 0.0                                 ! Conversion for FEV calc
   REAL    (KIND=RK) :: E, Div, HighE                                ! Local real variables
   REAL    (KIND=RK) :: Amb, AmbConcHome                             ! Local real variables
   ! Global constants: IK, LogU, LONG, LV_MS2, RK, SL_FM2, ST_MS2, YES 
   ! Global vars used: AlertThresh, DAvgDoseLev, DAvgExpLev, DebugLevel, DM1HDoseLev, DM1HExpLev,  
   !                   DM8HDoseLev, DM8HExpLev, DMEHDoseLev, DoDose, EventsPerson,  
   !                   H_EHDoseLev, MicroConc, NumEvents, NumHours, NumMicros, 
   !                   SAvgDoseLev, SAvgExpLev, TimeDoseLev, TimeExpLev
   ! Global vars set : EventSeq, Daily Exp, DailyDose, HourSeq, ProcLevel, ProcName
   !                   NChild, NActive, NActiveChild, NIll, NIllChild, HourConc
   !                   HourConc, HourTime, HourExp, HourEF, HourAmb
   ! Module constants: DAVGD, DAVGE, DM1HD, DM1HE, DM8HD, DM8HE, DMEHD,
   !                   H_EHD, SAVGD, SAVGE, TIMED, TIMEE
   ! Module vars used: NLev
   ! Module vars set : DAvgDose, DAvgExp, DM1HDose, DM1HExp, DM8HDose, DM8HExp, 
   !                   DMEHDose, H_EHDose, MSum, SAvgDose, SAvgExp, TimeDose, TimeExp
   ! Intrinsic procs : MAX, MAXVAL, PRINT, REAL, SUM, WRITE
   ! APEX procedures : CalcCOHb, Fatal
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "Exposure"                                  ! Procedure name for messages
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) ST_MS2, ProcName(ProcLevel)  ! Message for debugging    
   IF (Pol(PL)%InUnits==Pol(PL)%OutUnits) THEN                       ! If In = Out
      ExpF                 = Pol(PL)%PPMFact                         ! No overall conversion
   ELSEIF (Pol(PL)%InUnits==PPB .AND. Pol(PL)%OutUnits==PPM)  THEN   ! If In = PPB, Out = PPM
      ExpF                 = 0.001*Pol(PL)%PPMFact                   ! Convert PPB to PPM
   ELSEIF (Pol(PL)%InUnits==PPB .AND. Pol(PL)%OutUnits==UGM3) THEN   ! If In = PPB, Out = UGM
      ExpF                 = 0.001*Pol(PL)%PPMFact                   ! Convert PPB to UGM
   ELSEIF (Pol(PL)%InUnits==PPM .AND. Pol(PL)%OutUnits==PPB)  THEN   ! If In = PPM, Out = PPB
      ExpF                 = 1000*Pol(PL)%PPMFact                    ! Convert PPM to PPB
   ELSEIF (Pol(PL)%InUnits==PPM .AND. Pol(PL)%OutUnits==UGM3) THEN   ! If In = PPM, Out = UGM
      ExpF                 = Pol(PL)%PPMFact                         ! Convert PPM to UGM
   ELSEIF (Pol(PL)%InUnits==UGM3 .AND. Pol(PL)%OutUnits==PPB) THEN   ! If In = UGM, Out = PPB
      ExpF                 = 1/Pol(PL)%PPMFact*0.001                 ! Convert UGM to PPB
   ELSEIF (Pol(PL)%InUnits==UGM3 .AND. Pol(PL)%OutUnits==PPM) THEN   ! If In = UGM, Out = PPM
      ExpF                 = 1/Pol(PL)%PPMFact                       ! Convert UGM to PPM
   ENDIF                                                             ! End conversion logic   
   ExpFEV = Pol(PL)%PPMFact                                          ! Initializing ExpFEV
   IF (Pol(PL)%InUnits==PPB)  ExpFEV = 0.001*Pol(PL)%PPMFact         ! If input is in PPB
   IF (Pol(PL)%InUnits==UGM3) ExpFEV = 1/Pol(PL)%PPMFact             ! If input is in UGM3
   IF (PL==1) THEN                                                   ! If this the first pollutant
      HourConc(:,:,:) = 0                                            ! Init hourly conc for person 
   ENDIF                                                             ! End first pollutant logic          
   NOver = 0                                                         ! Initialize high CO event count
   TOver = 0                                                         ! Initialize high CO time
   HighE = 0.                                                        ! Initialize high concentration
   EventSeq(:)%Exposure = 0.                                         ! Initialize exposure per event
   TSSeq(:)%Exposure    = 0.                                         ! Initialize exposure per ts
   MSum(:)%Minutes      = 0                                          ! Initialize sum for minutes
   MSum(:)%MeanConc     = 0.                                         ! Initialize sum for mean conc
   MSum(:)%MaxConc      = 0.                                         ! Initialize max conc to zero
   HourTime(:,:)        = 0                                          ! Initialize time spent in mic
   LastLoc = 1                                                       ! Initializing last loc as home   
   DO I=1,NumHours                                                   ! Looping over hours to set event terms
      J = HourSeq(I)%FirstEvent                                      ! Index of first event in hour
      K = HourSeq(I)%LastEvent                                       ! Index of last event in hour
      EventSeq(J:K)%FEVE1 = HourSeq(I)%FEVE1                         ! Assign event FEV E1
      EventSeq(J:K)%FEVE2 = HourSeq(I)%FEVE2                         ! Assign event FEV E2
   ENDDO                                                             ! End loop over hours
   DO I=1,NumEvents                                                  ! Loop over events
      M = EventSeq(I)%Location                                       ! Event location
      D = EventSeq(I)%Duration                                       ! Event duration
      IF (EventSeq(I)%HomeWork==99) THEN                             ! If using last location
         Location = LastLoc+4                                        ! Set to last location
      ELSEIF (EventSeq(I)%HomeWork==4 .AND. Pol(PL)%RoadLast==YES) THEN ! If roadway, and last option...
         IF (LastLoc==1) Location=4                                  ! Set road home
         IF (LastLoc==2) THEN                                        ! If road work
             Location=7                                              ! Set road work         
             EventSeq(I)%HomeWork=7                                  ! Set HW variable
         ENDIF                                                       ! End road work conditional
      ELSE                                                           ! If not using last...
         Location = EventSeq(I)%HomeWork                             ! Set location of event
      ENDIF                                                          ! End location logic  
      IF (EventSeq(I)%HomeWork<=2) LastLoc = EventSeq(I)%HomeWork    ! Storing the last location
      E = MicroConc(EventSeq(I)%TS,M,Location) * ExpF                ! Exposure rate = conc * ExpF      
      IF (FEVFlag==1 .AND. INDEX(UP(Pol(PL)%Name),"O")==1) THEN      ! If FEV true, and pol is O3
         IF (Phys(P)%FEVSLP == -99) THEN                             ! If FEV values are undefined
            EventSeq(I)%FinalFEV = -99.999                           ! Store placeholder values
            EventSeq(I)%FinalX = -99.999                             ! Store placeholder values
         ELSE                                                        ! Else, if defined...
!             VE_BSA = (EventSeq(I)%VE/1000)/(0.007184*(Phys(P)%    &
!                Weight*0.45)**(0.425)*(Phys(P)%Height*2.54)**      &
!                (0.725))                                            ! Calculating VE/BSA like McDonnell 
             VE_BSA = (EventSeq(I)%VE/1000)/(Phys(P)%BSA)            ! Calculating VE/BSA using APEX BSA
             IF (I==1) THEN                                          ! If this is the first one
                EventSeq(I)%FinalX = ((MicroConc(EventSeq(I)%TS,M,  &
                  Location)*ExpFEV*(VE_BSA)**Phys(P)%Beta6)/        &
                  Phys(P)%Beta5)*(1-exp(-Phys(P)%Beta5*D))           ! Calculate X
             ELSE                                                    ! If not the first one
                EventSeq(I)%FinalX = EventSeq(I-1)%FinalX*          &
                  exp(-Phys(P)%Beta5*D) + ((MicroConc(EventSeq(I)%TS, &
                  M,Location)*ExpFEV*(VE_BSA)**Phys(P)%Beta6)/       &
                  Phys(P)%Beta5)*(1-exp(-Phys(P)%Beta5*D))           ! Calculate X a different way
             ENDIF                                                   ! End calculate X logic
             y = Phys(P)%FEVSLP * Person(P)%Age + Phys(P)%FEVINT     ! Calculating age term
             A = (Phys(P)%Beta1 + Phys(P)%Beta2*y + Phys(P)%Beta7*  &
                (Phys(P)%BMI-23.41))/(1 + Phys(P)%Beta4             &
                *exp(-Phys(P)%Beta3*EventSeq(I)%FinalX))             ! Calculating first term
             B = (Phys(P)%Beta1 + Phys(P)%Beta2*y + Phys(P)%Beta7*  &
                (Phys(P)%BMI-23.41))/(1 + Phys(P)%Beta4)             ! Calculating second term
             EventSeq(I)%FinalFEV = exp(Phys(P)%FEVU) * (A - B) +   &
                EventSeq(I)%FEVE1 + EventSeq(I)%FEVE2                ! Calculate FinalFEV
         ENDIF                                                       ! End missing value logic
      ENDIF                                                          ! End FEV logic
      EventSeq(I)%AmbHome = AQData(PL,EventSeq(I)%TS,Person(P)%HomeD)! Ambient conc in home distr       
      IF (EventSeq(I)%HomeWork==1) THEN                              ! If at home for this event                  
         Amb =AQData(PL,EventSeq(I)%TS,Person(P)%HomeD)              ! Ambient conc in home distr 
      ELSEIF(EventSeq(I)%HomeWork==2) THEN                           ! If at work during event
         Amb =AQData(PL,EventSeq(I)%TS,Person(P)%WorkD)              ! Ambient conc in work distr 
      ELSEIF(EventSeq(I)%HomeWork==4) THEN                           ! If at work during event
         Amb =AQData(PL,EventSeq(I)%TS,Person(P)%RoadD)              ! Ambient conc in road distr          
      ELSEIF(EventSeq(I)%HomeWork==5) THEN                           ! If near home sample
         Amb =AQData(PL,EventSeq(I)%TS,SList(NHLocs(                &
            EventSeq(I)%TS))%NumD)                                   ! Amb from near home array
      ELSEIF(EventSeq(I)%HomeWork==6) THEN                           ! If near work sample
         IF (NWLocs(EventSeq(I)%TS)/=-1) THEN                        ! If not a leaver
            Amb =AQData(PL,EventSeq(I)%TS,SList(NWLocs(             &
               EventSeq(I)%TS))%NumD)                                ! Amb from near work array
         ELSE                                                        ! Else if a leaver
            Amb =AQData(PL,EventSeq(I)%TS,-1)                        ! Amb from near work array
         ENDIF                                                       ! End leaver logic
      ELSEIF(EventSeq(I)%HomeWork==7) THEN                           ! If on road near work for event
         Amb =AQData(PL,EventSeq(I)%TS,Person(P)%RoadWD)             ! Ambient conc in road distr             
      ELSEIF(EventSeq(I)%HomeWork==99) THEN                          ! Set ambient for last loc
         IF (Location==5) Amb = AQData(PL,EventSeq(I)%TS,           &
            SList(NHLocs(EventSeq(I)%TS))%NumD)                      ! If home loc, home amb
         IF (Location==6 .AND. NWLocs(EventSeq(I)%TS)/=-1) THEN      ! If not a leaver...
            Amb = AQData(PL,EventSeq(I)%TS,SList(NWLocs(            &
               EventSeq(I)%TS))%NumD)                                ! If work loc, work amb
         ENDIF                                                       ! End nor leaver logic
         IF (Location==6 .AND. NWLocs(EventSeq(I)%TS)==-1) THEN      ! If a leaver...
            Amb = AQData(PL,EventSeq(I)%TS,-1)                       ! If work loc & leaver, work amb         
         ENDIF                                                       ! End leaver logic
      ELSE                                                           ! Otherwise in other place
         Amb =AQData(PL,EventSeq(I)%TS,0)                            ! Ambient conc in other distr            
      ENDIF                                                          ! End location logic      
      IF (E>POL(PL)%AlertThresh) THEN                                ! If person encounters high exp
         NOver = NOver+1                                             ! Increment high CO count
         TOver = TOver+D                                             ! Increment high CO time
         HighE = MAX(HighE,E)                                        ! Find maximum E encountered
      ENDIF                                                          ! End of high CO logic block
      IF (POL(PL)%N_TimeExpLev>0) THEN                               ! If TimeExp table is defined
         DO L=1,POL(PL)%N_TimeExpLev                                 ! Loop over levels for TimeExp 
            IF (E<TimeExpLev(PL,L)) EXIT                             ! Exit if E < level cutoff
         ENDDO                                                       ! Continue with next level
         IF (L==1) CALL Fatal(1)                                     ! This means exposure is negative
         TimeExp(M,L-1,PL) = TimeExp(M,L-1,PL)+REAL(D,LONG)          ! Row L-1 is where E belongs
      ENDIF                                                          ! End of TimeExp table logic     
      EventSeq(I)%Exposure   = E                                     ! Store exposure on time series
      EventSeq(I)%Amb        = Amb                                   ! Store amb conc on time series
      HourTime(EventSeq(I)%Hour,M) = HourTime(EventSeq(I)%Hour,M)+D  ! Store minutes spent in micro M
      HourConc(EventSeq(I)%Hour,PL,M) =                             &
         HourConc(EventSeq(I)%Hour,PL,M)+E*D/60                      ! Store hourly average conc in M
      IF (MSum(M)%MaxConc<E) MSum(M)%MaxConc = E                     ! Reset maximum if needed
      MSum(M)%Minutes  = MSum(M)%Minutes  + D                        ! Add minutes spent in micro M
      MSum(M)%MeanConc = MSum(M)%MeanConc + REAL(D,LONG)*E           ! Sum of time-weighted exposure
   ENDDO                                                             ! Continue with next event
   IF (NOver>0) THEN                                                 ! If this person had high exp
      AlertFlag=AlertFlag+1                                          ! Incr alert flag for high exp 
      PRINT*,       "High Exposure Level for Person ",              &
         P,NOver, TOver, HighE                                       ! Print warning message  
      IF (AlertFlag==1 .OR. P==NumProfiles) THEN                     ! If first high exposure
         WRITE(LogU,'(30X, A9, A25, A21, A20 )') "  Person  ",      &    
            "#Exposures>AlertThresh",                               &
            "Minutes>AlertThresh", "Max High Exposure"               ! Print warning header  
      ENDIF                                                          ! End first high exp logic
      WRITE(LogU,'(A30,I7,2X,I23,2X,I19,2X, F17.3 )')               &
         "High Exposure Level for Person ",                         & 
         P,NOver, TOver, HighE                                       ! Write message to log
      EventsPerson = YES                                             ! Save event output 
   ENDIF                                                             ! End of high CO logic block
   DO M=1,NumMicros                                                  ! Loop over micros
      Div = REAL(MAX(1,MSum(M)%Minutes))                             ! Time divisor (must not be zero) 
      MSum(M)%MeanConc = MSum(M)%MeanConc / Div                      ! Mean conc for micro M
   ENDDO                                                             ! Continue with next micro
   IF (DoHours==YES) THEN                                            ! If the hourly seq exists
      DO I=1,NumHours                                                ! Loop over hours
         J = HourSeq(I)%FirstEvent                                   ! Index of first event in hour
         K = HourSeq(I)%LastEvent                                    ! Index of last event in hour
         AmbConcHome = AQData(PL,EventSeq(K)%TS,Person(P)%HomeD)     ! Ambient conc in home district         
         HourSeq(I)%AmbHome = AmbConcHome                            ! Setting Conc in HourSeq array
         IF (SUM(EventSeq(J:K)%Duration)/=60) CALL Fatal(2)          ! Check that hour has 60 minutes
         HourSeq(I)%Exposure = SUM(EventSeq(J:K)%Exposure*          &
            REAL(EventSeq(J:K)%Duration))/60.                        ! Time-weighted average exposure
         HourSeq(I)%Amb = SUM(EventSeq(J:K)%Amb*                    &
            REAL(EventSeq(J:K)%Duration))/60.                        ! Time-weighted ambient conc            
         HourSeq(I)%AmbHome = SUM(EventSeq(J:K)%AmbHome*            &
            REAL(EventSeq(J:K)%Duration))/60.                        ! Time-weighted ambient conc              
         IF (HourSeq(I)%AmbHome == 0) THEN                           ! When ambient home conc 0
            HourSeq(I)%HomeEF = -999                                 ! If ambient conc is 0, HomeEF undef
         ELSE                                                        ! Otherwise homeEF is defined  
            HourSeq(I)%HomeEF =  HourSeq(I)%Exposure/HourSeq(I)%AmbHome ! Hourly exposure factor
         ENDIF                                                       ! End home ambient logic
         IF (HourSeq(I)%Amb == 0) THEN                               ! When ambient conc 0       
            HourSeq(I)%EF = -999                                     ! If ambient conc is 0, EF undef
         ELSE                                                        ! Otherwise EF is defined  
            HourSeq(I)%EF =  HourSeq(I)%Exposure/HourSeq(I)%Amb      ! Hourly exposure factor
         ENDIF                                                       ! End ambient conc logic
         IF (I<=8) THEN                                              ! First 8 hours use special logic 
            HourSeq(I)%Run8Exp  = SUM(HourSeq(1:I)%Exposure)/I       ! Running 8-hour average for exp 
            HourSeq(I)%Run8AmbHome  = SUM(HourSeq(1:I)%AmbHome)/I    ! Running 8-hour average for amb
         ELSE                                                        ! If beyond first 8 hours
            HourSeq(I)%Run8Exp  = SUM(HourSeq(I-7:I)%Exposure)/8.0   ! Appropriate average
            HourSeq(I)%Run8AmbHome  = SUM(HourSeq(I-7:I)%AmbHome)/8.0 ! Appropriate average
         ENDIF                                                       ! End of running 8-hour logic
         IF (FEVFlag==1 .AND. INDEX(UP(Pol(PL)%Name),"O")==1) THEN   ! If FEV true, and pol is O3      
            HourSeq(I)%dFEV1max = MAXVAL(EventSeq(J:K)%FinalFEV)     ! Find hourly max 1-hour exposure         
         ENDIF                                                       ! End %dFEV1 logic
      ENDDO                                                          ! Continue with next hour
   ENDIF                                                             ! End hourly logic
   T =  TSMinutes                                                    ! Local copy of minutes/timestep
   DO I=1,NumTS                                                      ! Loop over ts
      J = TSSeq(I)%FirstEvent                                        ! Index of first event in ts
      K = TSSeq(I)%LastEvent                                         ! Index of last event in ts     
      IF(SUM(EventSeq(J:K)%Duration)/=TSMinutes) CALL Fatal(3)       ! Check that ts has right # mins
      TSSeq(I)%Exposure = SUM(EventSeq(J:K)%Exposure*               &
         REAL(EventSeq(J:K)%Duration))/REAL(T)                       ! Time-weighted average exposure
      TSSeq(I)%Amb = SUM(EventSeq(J:K)%Amb*                         &
         REAL(EventSeq(J:K)%Duration))/REAL(T)                       ! Time-weighted ambient conc  
      IF (TSSeq(I)%Amb == 0) THEN                                    ! When ambient is 0
          TSSeq(I)%EF = -999                                         ! If ambient conc is 0, EF undef
      ELSE                                                           ! Otherwise EF is defined
         TSSeq(I)%EF =  TSSeq(I)%Exposure/TSSeq(I)%Amb               ! Timestep exposure factor
      ENDIF                                                          ! End ambient conc logic    
      IF (FEVFlag==1 .AND. INDEX(UP(Pol(PL)%Name),"O")==1) THEN      ! If FEV true, and pol is O3      
         TSSeq(I)%dFEV1max = MAXVAL(EventSeq(J:K)%FinalFEV)          ! Find TS max 1-hour exposure         
      ENDIF                                                          ! End %dFEV1 logic      
   ENDDO                                                             ! Continue with next timestep
   CALL SummarizeExposure(P,PL)                                      ! Summarize exposure for tables
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE Exposure                                              ! Return to main APEX program  

SUBROUTINE SummarizeExposure(P, PL)                                  ! Called from Exposure
   INTEGER (KIND=IK), INTENT(IN) :: P, PL                            ! Input variables (person, pollut)
   INTEGER (KIND=IK) :: I, J, L, M, N, TS                            ! Local integer variables
   REAL    (KIND=RK) :: E(3), X, EVR, EVRModerate, Y                 ! Local real variables
   REAL    (KIND=RK) :: EVRHeavy, ExpM, ExpH                         ! Local real variables
   LOGICAL (KIND=1)  :: Ill,Child, Active, Employed                  ! Shorter form for Ill
   ! Global constants: IK, LogU, LONG, LV_MS2, RK, SL_FM2, ST_MS2, YES 
   ! Global vars used: AlertThresh, DAvgDoseLev, DAvgExpLev, DebugLevel, DM1HDoseLev, DM1HExpLev,  
   !                   DM8HDoseLev, DM8HExpLev, DMEHDoseLev, DoDose, EventsPerson,  
   !                   H_EHDoseLev, MicroConc, NumEvents, NumHours, NumMicros, 
   !                   SAvgDoseLev, SAvgExpLev, TimeDoseLev, TimeExpLev
   ! Global vars set : EventSeq, Daily Exp, DailyDose, HourSeq, ProcLevel, ProcName
   !                   NChild, NActive, NActiveChild, NIll, NIllChild, HourConc
   !                   HourConc, HourTime, HourExp, HourEF, HourAmb
   ! Module constants: DAVGD, DAVGE, DM1HD, DM1HE, DM8HD, DM8HE, DMEHD,
   !                   H_EHD, SAVGD, SAVGE, TIMED, TIMEE
   ! Module vars used: NLev
   ! Module vars set : DAvgDose, DAvgExp, DM1HDose, DM1HExp, DM8HDose, DM8HExp, 
   !                   DMEHDose, H_EHDose, MSum, SAvgDose, SAvgExp, TimeDose, TimeExp
   ! Intrinsic procs : MAX, MAXVAL, PRINT, REAL, SUM, WRITE
   ! APEX procedures : Fatal
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "SummarizeExposure"                         ! Procedure name for messages
   Child = (Person(P)%Age>=ChildMin .AND. Person(P)%Age<=ChildMax)   ! 'Child' is true or false  
   Active = (Person(P)%PAI>=ActivePAI)                               ! 'Active' is true or false  
   Employed = (Person(P)%Employed==YES)                              ! 'Employed' is true or false   
   Ill    = (Phys(P)%Ill==Yes)                                       ! 'Ill' is true or false  
   TS = TimeStepsPerDay                                              ! Local copy of TimestepsPerDay   
   IF (PL==1) THEN                                                   ! If this the first pollutant
      IF (Child)  NChild                   = NChild+1                ! Update number of children 
      IF (Active) NActive                  = NActive+1               ! Update number of actives  
      IF (Active .AND. Child) NActiveChild = NActiveChild+1          ! Update number of actv children  
      IF (Ill)    NIll                     = NIll+1                  ! Update number of ill
      IF (Ill .AND. Child) NIllChild       = NIllChild+1             ! Update number of ill children
      IF (Employed) NEmployed              = NEmployed+1             ! Update number of employed     
   ENDIF                                                             ! End first pollutant logic          
   DO I=1,NumDays                                                    ! Loop over days in simulation
      IF (FEVFlag==1 .AND. INDEX(UP(Pol(PL)%Name),"O")==1) THEN      ! If FEV true, and pol is O3      
         DailyExp(I)%dFEV1max = MAXVAL(HourSeq(24*I-23:24*I)%dFEV1max) ! Find daily FEV1 max 
      ENDIF                                                          ! End %dFEV1 logic   
      IF (DoHours==YES) THEN                                         ! If 1,8 hour tables needed
         X        = MAXVAL(HourSeq(24*I-23:24*I)%Exposure)           ! Find daily max 1-hour exposure
         DailyExp(I)%Max1Exp = X                                     ! Store daily max 1-hour exposure
         EVRModerate = ModEVR1                                       ! Moderate 1-Hour EVR (l/min-m^2)          
         EVRHeavy    = HeavyEVR1                                     ! Heavy 1-Hour EVR (l/min-m^2)     
         ExpM = -1                                                   ! Init max exp at mod EVR
         ExpH = -1                                                   ! Init max exp at heavy EVR
         DO J=1,24                                                   ! Loop over hours in day
            EVR = HourSeq(24*I-23+(J-1))%EVR                         ! Current hourly avg EVR       
            IF (EVR >= EVRModerate .AND. EVR < EVRHeavy)            &        
               ExpM=MAX(HourSeq(24*I-23+(J-1))%Exposure, ExpM)       ! Update max at mod EVR
            IF (EVR >= EVRHeavy)                                    &        
               ExpH=MAX(HourSeq(24*I-23+(J-1))%Exposure, ExpH)       ! Update max at heavy EVR
         ENDDO                                                       ! End loop over days    
         E(1) = X                                                    ! MAX 1H Exposure
         E(2) = ExpM                                                 ! MAX 1H Exposure during moderate
         E(3) = ExpH                                                 ! MAX 1H Exposure during heavy
         DO L=1,POL(PL)%N_DM1HExpLev                                 ! Loop over levels in DM1HE table
            DO N=0,2                                                 ! Do 3X: all, moderate, heavy    
               M=N*7                                                 ! For indexing tables   
               IF (E(N+1)>=DM1HExpLev(PL,L)) THEN                    ! If exposure level exceeded
                  DM1HExp(P,L,1+M,PL)=DM1HExp(P,L,1+M,PL)+1          ! Increment "all" group 
                  IF (Child) DM1HExp(P,L,2+M,PL)                    &    
                     = DM1HExp(P,L,2+M,PL)+1                         ! Increment "child" group     
                  IF (Active) DM1HExp(P,L,3+M,PL)                   &
                     = DM1HExp(P,L,3+M,PL)+1                         ! Increment "active" group      
                  IF (Active.AND.Child) DM1HExp(P,L,4+M,PL)         &
                     = DM1HExp(P,L,4+M,PL)+1                         ! Increment "active child" group
                  IF (Ill) DM1HExp(P,L,5+M,PL)                      &
                     = DM1HExp(P,L,5+M,PL)+1                         ! Increment "ill" group
                  IF (Ill .AND. Child) DM1HExp(P,L,6+M,PL)          &
                     = DM1HExp(P,L,6+M,PL)+1                         ! Increment "ill child" group 
                  IF (Employed) DM1HExp(P,L,7+M,PL)          &
                     = DM1HExp(P,L,7+M,PL)+1                         ! Increment "employed" group
               ENDIF                                                 ! End level exceeded logic 
            ENDDO                                                    ! End loop over exposure types
         ENDDO                                                       ! Continue with next table level
         X = MAXVAL(HourSeq(24*I-23:24*I)%Run8Exp)                   ! Find daily max 8-hour exposure
         Y = MAXLOC(HourSeq(24*I-23:24*I)%Run8Exp,dim=1)             ! Find location of daily max
         DailyExp(I)%Max8EC = X/HourSeq(Y+24*(I-1))%Run8AmbHome      ! Ambient average at daily8max         
         DailyExp(I)%Max8Exp = X                                     ! Store daily max 8-hour exposure
         EVRModerate = ModEVR8                                       ! Moderate 8-Hour EVR (l/min-m^2)   
         EVRHeavy    = HeavyEVR8                                     ! Heavy 8-Hour EVR (l/min-m^2)
         ExpM = -1.                                                  ! Init max exp at mod EVR
         ExpH = -1.                                                  ! Init max exp at heavy EVR
         DO J=1,24                                                   ! Loop over hours in day
            EVR = HourSeq(24*I-23+(J-1))%Run8EVR                     ! Current 8 hour avg EVR 
            IF (EVR >= EVRModerate .AND. EVR < EVRHeavy)            &        
               ExpM=MAX(HourSeq(24*I-23+(J-1))%Run8Exp, ExpM)        ! Update max at mod EVR
            IF ( EVR >= EVRHeavy)                                   &        
               ExpH=MAX(HourSeq(24*I-23+(J-1))%Run8Exp, ExpH)        ! Update max at heavy EVR
         ENDDO                                                       ! End loop over hours in day       
         E(1) = X                                                    ! MAX 8H Exposure
         E(2) = ExpM                                                 ! MAX 8H Exposure during moderate
         E(3) = ExpH                                                 ! MAX 8H Exposure during heavy
         DO L=1,POL(PL)%N_DM8HExpLev                                 ! Loop over levels in DM8HE table
            DO N=0,2                                                 ! Do 3X: all, moderate, heavy    
               M=N*7                                                 ! For indexing tables   
               IF (E(N+1)>=DM8HExpLev(PL,L)) THEN                    ! If exposure level exceeded
                  DM8HExp(P,L,1+M,PL)=DM8HExp(P,L,1+M,PL)+1          ! Increment "all" group 
                  IF (Child) DM8HExp(P,L,2+M,PL)                    &    
                     = DM8HExp(P,L,2+M,PL)+1                         ! Increment "child" group
                  IF (Active) DM8HExp(P,L,3+M,PL)                   &
                     = DM8HExp(P,L,3+M,PL)+1                         ! Increment "active" group
                  IF (Active.AND.Child) DM8HExp(P,L,4+M,PL)         &
                     = DM8HExp(P,L,4+M,PL)+1                         ! Increment "active child" group
                  IF (Ill) DM8HExp(P,L,5+M,PL)                      &
                     = DM8HExp(P,L,5+M,PL)+1                         ! Increment "ill" group
                  IF (Ill .AND. Child) DM8HExp(P,L,6+M,PL)          &
                     = DM8HExp(P,L,6+M,PL)+1                         ! Increment "ill child" group
                  IF (Employed) DM8HExp(P,L,7+M,PL)          &
                     = DM8HExp(P,L,7+M,PL)+1                         ! Increment "employed" group
               ENDIF                                                 ! End level exceeded logic 
            ENDDO                                                    ! End loop over exposure types
         ENDDO                                                       ! Continue with next table level
      ENDIF                                                          ! End hourly logic    
      X = MAXVAL(TSSeq(TS*I-(TS-1):TS*I)%Exposure)                   ! Find daily max ts exposure
      DailyExp(I)%MaxTSExp = X                                       ! Store daily max ts exposure
      EVRModerate = ModEVRTS                                         ! Moderate ts EVR (l/min-m^2)          
      EVRHeavy    = HeavyEVRTS                                       ! Heavy ts EVR (l/min-m^2)     
      ExpM = -1                                                      ! Init max exp at mod EVR
      ExpH = -1                                                      ! Init max exp at heavy EVR
      DO J=1,TS                                                      ! Loop over ts in day
         EVR = TSSeq(TS*I-(TS-1)+(J-1))%EVR                          ! Current ts avg EVR       
         IF (EVR >= EVRModerate .AND. EVR < EVRHeavy)               &          
            ExpM=MAX(TSSeq(TS*I-(TS-1)+(J-1))%Exposure, ExpM)        ! Update max at mod EVR
         IF (EVR >= EVRHeavy)                                       &        
            ExpH=MAX(TSSeq(TS*I-(TS-1)+(J-1))%Exposure, ExpH)        ! Update max at heavy EVR
      ENDDO                                                          ! End loop over days    
      E(1) = X                                                       ! MAX TS Exposure
      E(2) = ExpM                                                    ! MAX TS Exposure during moderate
      E(3) = ExpH                                                    ! MAX TS Exposure during heavy
      DO L=1,POL(PL)%N_DMTSExpLev                                    ! Loop over levels in DMTSE table
         DO N=0,2                                                    ! Do 3X: all, moderate, heavy    
            M=N*7                                                    ! For indexing tables   
            IF (E(N+1)>=DMTSExpLev(PL,L)) THEN                       ! If exposure level exceeded
               DMTSExp(P,L,1+M,PL)=DMTSExp(P,L,1+M,PL)+1             ! Increment "all" group 
               IF (Child) DMTSExp(P,L,2+M,PL)                       &    
                  = DMTSExp(P,L,2+M,PL)+1                            ! Increment "child" group     
               IF (Active) DMTSExp(P,L,3+M,PL)                      &
                  = DMTSExp(P,L,3+M,PL)+1                            ! Increment "active" group      
               IF (Active.AND.Child) DMTSExp(P,L,4+M,PL)            &
                  = DMTSExp(P,L,4+M,PL)+1                            ! Increment "active child" group
               IF (Ill) DMTSExp(P,L,5+M,PL)                         &
                  = DMTSExp(P,L,5+M,PL)+1                            ! Increment "ill" group
               IF (Ill .AND. Child) DMTSExp(P,L,6+M,PL)             &
                  = DMTSExp(P,L,6+M,PL)+1                            ! Increment "ill child" group   
               IF (Employed) DMTSExp(P,L,7+M,PL)                    &
                  = DMTSExp(P,L,7+M,PL)+1                            ! Increment "employed" group
            ENDIF                                                    ! End level exceeded logic 
         ENDDO                                                       ! End loop over exposure types
      ENDDO                                                          ! Continue with next table level
      X = SUM(TSSeq(TS*I-(TS-1):TS*I)%Exposure)/TS                   ! Average daily exposure
      DO L=1,POL(PL)%N_DAvgExpLev                                    ! Loop over levels in DAvg table
         IF (X>=DAvgExpLev(PL,L)) DAvgExp(P,L,PL)=DAvgExp(P,L,PL)+1  ! Increment levels exceeded by X
      ENDDO                                                          ! Continue with next table level
      DailyExp(I)%AvgExp = X                                         ! Store daily average exposure
   ENDDO                                                             ! Continue with next day
   EVRModerate = ModEVRTS                                            ! Moderate ts EVR (l/min-m^2)          
   EVRHeavy    = HeavyEVRTS                                          ! Heavy ts EVR (l/min-m^2)     
   DO I=1, NumTS                                                     ! Loop over timesteps
      EVR = TSSeq(I)%EVR                                             ! Current timestep EVR       
      ExpM = -1                                                      ! Reset exp at mod EVR
      ExpH = -1                                                      ! Reset exp at heavy EVR
      X = TSSeq(I)%Exposure                                          ! Timestep exposure 
      IF (EVR >= EVRModerate .AND. EVR < EVRHeavy)                  &          
         ExpM=TSSeq(I)%Exposure                                      ! Update val at mod EVR
      IF (EVR >= EVRHeavy) ExpH=TSSeq(I)%Exposure                    ! Update max at heavy EVR
      E(1) = X                                                       ! TS Exposure
      E(2) = ExpM                                                    ! TS Exposure during moderate
      E(3) = ExpH                                                    ! TS Exposure during heavy
      DO L=1,POL(PL)%N_TSExpLev                                      ! Loop over TSExp table levels
         DO N=0,2                                                    ! Do 3X: all, moderate, heavy    
            M=N*7                                                    ! For indexing tables   
            IF (E(N+1)>=TSExpLev(PL,L)) THEN                         ! If exposure level exceeded
               ALLTSExp(P,L,1+M,PL)=ALLTSExp(P,L,1+M,PL)+1           ! Increment "all" group 
               IF (Child) ALLTSExp(P,L,2+M,PL)                      &    
                  = ALLTSExp(P,L,2+M,PL)+1                           ! Increment "child" group     
               IF (Active) ALLTSExp(P,L,3+M,PL)                     &
                  = ALLTSExp(P,L,3+M,PL)+1                           ! Increment "active" group      
               IF (Active.AND.Child) ALLTSExp(P,L,4+M,PL)           &
                  = ALLTSExp(P,L,4+M,PL)+1                           ! Increment "active child" group
               IF (Ill) ALLTSExp(P,L,5+M,PL)                        &
                  = ALLTSExp(P,L,5+M,PL)+1                           ! Increment "ill" group
               IF (Ill .AND. Child) ALLTSExp(P,L,6+M,PL)            &
                  = ALLTSExp(P,L,6+M,PL)+1                           ! Increment "ill child" group    
               IF (Employed) ALLTSExp(P,L,7+M,PL)                   &
                  = ALLTSExp(P,L,7+M,PL)+1                           ! Increment "employed" group
            ENDIF                                                    ! End level exceeded logic 
         ENDDO                                                       ! End loop over exposure types
      ENDDO                                                          ! Continue with next table level  
   ENDDO                                                             ! End loop over timesteps
   X = SUM(DailyExp(1:NumDays)%AvgExp)/NumDays                       ! Average exposure over simulation
   DO L=1,POL(PL)%N_SAvgExpLev                                       ! Loop over SAvgExp table levels
      IF (X>=SAvgExpLev(PL,L)) SAvgExp(P,L,PL)=SAvgExp(P,L,PL)+1     ! Increment levels exceeded by X
   ENDDO                                                             ! Continue with next table level  
   AvgExp(P,PL)=SUM(TSSeq(:)%Exposure)/REAL(NumTS)                   ! Save avg exposure for person
   MaxExp(P,PL)=MAXVAL(TSSeq(:)%Exposure)                            ! Save max exposure for person
   DO I=1,NumEvents                                                  ! Loop over events
      EventExp(I,PL) = EventSeq(I)%Exposure                          ! Save event exp for person
      EventAmb(I,PL) = EventSeq(I)%Amb                               ! Save event amb for person
      EventAmbHome(I,PL) = EventSeq(I)%AmbHome                       ! Save event home amb for person   
   ENDDO                                                             ! End loop over events
   IF (DoHours==YES) THEN                                            ! If hour data needed
      DO I=1,NumHours                                                ! Loop over hours
         HourExp(I,PL) = HourSeq(I)%Exposure                         ! Save hourly exp for person
         HourEF(I,PL)  = HourSeq(I)%EF                               ! Save hourly EF for person
         HourAmb(I,PL) = HourSeq(I)%Amb                              ! Save hourly ambient
         HourHomeAmb(I,PL)= HourSeq(I)%AmbHome                       ! Save hourly home ambient
         HourHomeEF(I,PL) = HourSeq(I)%HomeEF                        ! Save hourly home EF for person
      ENDDO                                                          ! End loop over events
   ENDIF                                                             ! End hourly logic
   DO I=1,NumTS                                                      ! Loop over ts
      TSExp(I,PL) = TSSeq(I)%Exposure                                ! Save ts exp for person
      TSEF(I,PL)  = TSSeq(I)%EF                                      ! Save ts EF for person
      TSAmb(I,PL) = TSSeq(I)%Amb                                     ! Save ts ambient exp
   ENDDO                                                             ! End loop over ts
   DO I=1,NumDays                                                    ! Loop over days
      DayAvgExp(I,PL)  = DailyExp(I)%AvgExp                          ! Save daily avg exp
      DayMax1Exp(I,PL) = DailyExp(I)%Max1Exp                         ! Save daily max 1 exp
      DayMax8Exp(I,PL) = DailyExp(I)%Max8Exp                         ! Save daily max 8 exp
      DayMaxTSExp(I,PL)= DailyExp(I)%MaxTSExp                        ! Save daily max 8 exp
      DayMax8EC(I,PL)  = DailyExp(I)%Max8EC                          ! Save daily max 8 E/C
   ENDDO                                                             ! End loop over events
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE SummarizeExposure                                     ! Return to main APEX program  

SUBROUTINE Dose(P,PL)
   INTEGER (KIND=IK), INTENT(IN) :: P, PL                            ! Input variables (person, pollut)
   INTEGER (KIND=IK) :: I,L,J,K                                      ! Local integer variables
   REAL    (KIND=RK) :: X                                            ! Local real variables
   IF (Pol(PL)%DoDose==NO) RETURN                                    ! Not doing dose for this pollut
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "Dose"                                      ! Procedure name for messages
   EventSeq(:)%Dose     = 0.                                         ! Initialize mean  dose per event
   EventSeq(:)%FDose    = 0.                                         ! Initialize final dose per event
   HourSeq(:)%Dose      = 0.                                         ! Initialize mean  dose per hour
   HourSeq(:)%FDose     = 0.                                         ! Initialize final dose per hour
   HourSeq(:)%Run8Exp   = 0.                                         ! Initialize running 8-hour exp
   HourSeq(:)%Run8Dose  = 0.                                         ! Initialize running 8-hour dose
   HourSeq(:)%Run8AmbHome   = 0.                                     ! Initialize running 8-hour amb   
   IF (Pol(PL)%Name=="CO") THEN                                      ! If pollutant is CO
      CALL CalcCOHb(P,PL)                                            ! Calculate blood levels for CO
   ELSEIF (Pol(PL)%PM==YES) THEN                                     ! If this is a particle
      CALL PMDose(P,PL)                                              ! Call the PM dose routine
   ELSE                                                              ! For all other pollutants
      CALL GeneralDose(P, PL)                                        ! Calulate a general dose
   ENDIF                                                             ! End pollutant logic  
   IF (POL(PL)%N_TimeDoseLev>0) THEN                                 ! If TimeDose table is defined 
      DO I=1,NumEvents                                               ! Loop over events
         DO L=1,POL(PL)%N_TimeDoseLev                                ! Loop over levels in table    
            IF (EventSeq(I)%Dose<TimeDoseLev(PL,L)) EXIT             ! Exit if Dose < level cutoff
         ENDDO                                                       ! Continue with next level 
         TimeDose(P,L-1,PL) = TimeDose(P,L-1,PL)+                   &  
            REAL(EventSeq(I)%Duration,LONG)                          ! Accumulate time at level L-1
      ENDDO                                                          ! Continue with next event
   ENDIF                                                             ! End of TimeDose logic block
   IF (DoHours==YES) THEN                                            ! If the hourly seq exists
      DO I=1,NumHours                                                ! Loop over hours
         J = HourSeq(I)%FirstEvent                                   ! Index of first event in hour
         K = HourSeq(I)%LastEvent                                    ! Index of last event in hour
         HourSeq(I)%Dose = SUM(EventSeq(J:K)%Dose*                  &
            REAL(EventSeq(J:K)%Duration))/60.                        ! Time-weighted average dose 
         HourSeq(I)%IntakeDose = SUM(EventSeq(J:K)%IntakeDose*      &
            REAL(EventSeq(J:K)%Duration))/60.                        ! Time-weighted average dose 
         HourSeq(I)%DepDose = SUM(EventSeq(J:K)%DepDose)             ! Sum of deposited dose 
         HourSeq(I)%FDose = EventSeq(K)%FDose                        ! End-of-hour dose
         IF (POL(PL)%N_H_EHDoseLev >0) THEN                          ! If H_EHDose table is defined
            X = HourSeq(I)%FDose                                     ! Temporary variable for H_EHDose
            DO L=1,POL(PL)%N_H_EHDoseLev                             ! Loop over H_EHDose levels
               IF (X>=H_EHDoseLev(PL,L)) H_EHDose(P,L,PL)           &
                  =H_EHDose(P,L,PL)+1                                ! Increment levels exceeded by X
            ENDDO                                                    ! Continue with next table level 
         ENDIF                                                       ! End of H_EHDose logic block 
         IF (I<=8) THEN                                              ! First 8 hours use special logic 
            HourSeq(I)%Run8Dose = SUM(HourSeq(1:I)%Dose)/I           ! Running 8-hour average for dose
         ELSE                                                        ! If beyond first 8 hours
            HourSeq(I)%Run8Dose = HourSeq(I-1)%Run8Dose +           &
               (HourSeq(I)%Dose-HourSeq(I-8)%Dose)/8.                ! Add new hour and drop off old
         ENDIF                                                       ! End of running 8-hour logic
      ENDDO                                                          ! End loop over hours
   ENDIF                                                             ! End hourly logic
   DO I=1,NumTS                                                      ! Loop over ts
      J = TSSeq(I)%FirstEvent                                        ! Index of first event in ts
      K = TSSeq(I)%LastEvent                                         ! Index of last event in ts
      IF(SUM(EventSeq(J:K)%Duration)/=TSMinutes) CALL Fatal(1)       ! Check that ts has right # mins
      TSSeq(I)%Dose = SUM(EventSeq(J:K)%Dose*                       &
         REAL(EventSeq(J:K)%Duration))/REAL(T)                       ! Time-weighted dose
      TSSeq(I)%IntakeDose = SUM(EventSeq(J:K)%IntakeDose*           &
         REAL(EventSeq(J:K)%Duration))/REAL(T)                       ! Time-weighted intake dose
      TSSeq(I)%FDose = EventSeq(K)%FDose                             ! End-of-timestep dose
      TSSeq(I)%DepDose = SUM(EventSeq(J:K)%DepDose)                  ! Sum of deposited dose
   ENDDO                                                             ! Continue with next timestep
   CALL SummarizeDose(P,PL)                                          ! Call SummarizeDose routine
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE Dose                                                  ! Return to main APEX program

SUBROUTINE SummarizeDose(P,PL)
   INTEGER (KIND=IK), INTENT(IN) :: P, PL                            ! Input variables (person, pollut)
   INTEGER (KIND=IK) :: I,L,J,N,M,TS                                 ! Local integer variables
   REAL    (KIND=RK) :: X, D(3)                                      ! Local real variables
   REAL    (KIND=RK) :: DoseM, DoseH, EVR, EVRModerate, EVRHeavy     ! Local real variables
   LOGICAL (KIND=1)  :: Ill,Child, Active, Employed                  ! Shorter form for Ill
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "SummarizeDose"                             ! Procedure name for messages
   Child = (Person(P)%Age>=ChildMin .AND. Person(P)%Age<=ChildMax)   ! 'Child' is true or false  
   Active = (Person(P)%PAI>=ActivePAI)                               ! 'Active' is true or false
   Employed = (Person(P)%Employed==YES)                              ! 'Employed' is true or false
   Ill    = (Phys(P)%Ill==Yes)                                       ! 'Ill' is true or false  
   TS = TimeStepsPerDay                                              ! Local copy of TimestepsPerDay   
   DO I=1,NumDays                                                    ! Loop over days in simulation
      IF (DoHours==YES) THEN                                         ! If 1,8 hour tables needed
         X        = MAXVAL(HourSeq(24*I-23:24*I)%Dose)               ! Find daily max 1-hour dose
         DailyDose(I)%Max1Dose = X                                   ! Store daily max 1-hour dose
         EVRModerate = ModEVR1                                       ! Moderate 1-Hour EVR (l/min-m^2)          
         EVRHeavy    = HeavyEVR1                                     ! Heavy 1-Hour EVR (l/min-m^2)     
         DoseM = -1                                                  ! Init max dose at mod EVR
         DoseH = -1                                                  ! Init max dose at heavy EVR
         DO J=1,24                                                   ! Loop over hours in day
            EVR = HourSeq(24*I-23+(J-1))%EVR                         ! Current hourly avg EVR       
            IF (EVR >= EVRModerate .AND. EVR < EVRHeavy)            &        
               DoseM=MAX(HourSeq(24*I-23+(J-1))%Dose, DoseM)         ! Update max at mod EVR
            IF (EVR >= EVRHeavy)                                    &        
               DoseH=MAX(HourSeq(24*I-23+(J-1))%Dose, DoseH)         ! Update max at heavy EVR
         ENDDO                                                       ! End loop over days    
         D(1) = X                                                    ! MAX 1H Dose
         D(2) = DoseM                                                ! MAX 1H Dose during moderate
         D(3) = DoseH                                                ! MAX 1H Dose during heavy
         DO L=1,POL(PL)%N_DM1HDoseLev                                ! Loop over levels in DM1HE table
            DO N=0,2                                                 ! Do 3X: all, moderate, heavy    
               M=N*7                                                 ! For indexing tables   
               IF (D(N+1)>=DM1HDoseLev(PL,L)) THEN                   ! If Dose level exceeded
                  DM1HDose(P,L,1+M,PL)=DM1HDose(P,L,1+M,PL)+1        ! Increment "all" group 
                  IF (Child) DM1HDose(P,L,2+M,PL)                    &    
                     = DM1HDose(P,L,2+M,PL)+1                        ! Increment "child" group     
                  IF (Active) DM1HDose(P,L,3+M,PL)                   &
                     = DM1HDose(P,L,3+M,PL)+1                        ! Increment "active" group      
                  IF (Active.AND.Child) DM1HDose(P,L,4+M,PL)         &
                     = DM1HDose(P,L,4+M,PL)+1                        ! Increment "active child" group
                  IF (Ill) DM1HDose(P,L,5+M,PL)                      &
                     = DM1HDose(P,L,5+M,PL)+1                        ! Increment "ill" group
                  IF (Ill .AND. Child) DM1HDose(P,L,6+M,PL)          &
                     = DM1HDose(P,L,6+M,PL)+1                        ! Increment "ill child" group 
                  IF (Employed) DM1HDose(P,L,7+M,PL)                 &
                     = DM1HDose(P,L,7+M,PL)+1                        ! Increment "employed" group
               ENDIF                                                 ! End level exceeded logic 
            ENDDO                                                    ! End loop over dose types
         ENDDO                                                       ! Continue with next table level
         X        = MAXVAL(HourSeq(24*I-23:24*I)%Run8Dose)           ! Find daily max 1-hour dose
         DailyDose(I)%Max8Dose = X                                   ! Store daily max 8-hour dose
         EVRModerate = ModEVR8                                       ! Moderate 1-Hour EVR (l/min-m^2)          
         EVRHeavy    = HeavyEVR8                                     ! Heavy 1-Hour EVR (l/min-m^2)     
         DoseM = -1                                                  ! Init max dose at mod EVR
         DoseH = -1                                                  ! Init max dose at heavy EVR
         DO J=1,24                                                   ! Loop over hours in day
            EVR = HourSeq(24*I-23+(J-1))%EVR                         ! Current hourly avg EVR       
            IF (EVR >= EVRModerate .AND. EVR < EVRHeavy)            &        
               DoseM=MAX(HourSeq(24*I-23+(J-1))%Run8Dose, DoseM)     ! Update max at mod EVR
            IF (EVR >= EVRHeavy)                                    &        
               DoseH=MAX(HourSeq(24*I-23+(J-1))%Run8Dose, DoseH)     ! Update max at heavy EVR
         ENDDO                                                       ! End loop over hours
         D(1) = X                                                    ! MAX 8H Dose
         D(2) = DoseM                                                ! MAX 8H Dose during moderate
         D(3) = DoseH                                                ! MAX 8H Dose during heavy
         DO L=1,POL(PL)%N_DM8HDoseLev                                ! Loop over levels in DM1HE table
            DO N=0,2                                                 ! Do 3X: all, moderate, heavy    
               M=N*7                                                 ! For indexing tables   
               IF (D(N+1)>=DM8HDoseLev(PL,L)) THEN                   ! If Dose level exceeded
                  DM8HDose(P,L,1+M,PL)=DM8HDose(P,L,1+M,PL)+1        ! Increment "all" group 
                  IF (Child) DM8HDose(P,L,2+M,PL)                    &    
                     = DM8HDose(P,L,2+M,PL)+1                        ! Increment "child" group     
                  IF (Active) DM8HDose(P,L,3+M,PL)                   &
                     = DM8HDose(P,L,3+M,PL)+1                        ! Increment "active" group      
                  IF (Active.AND.Child) DM8HDose(P,L,4+M,PL)         &
                     = DM8HDose(P,L,4+M,PL)+1                        ! Increment "active child" group
                  IF (Ill) DM8HDose(P,L,5+M,PL)                      &
                     = DM8HDose(P,L,5+M,PL)+1                        ! Increment "ill" group
                  IF (Ill .AND. Child) DM8HDose(P,L,6+M,PL)          &
                     = DM8HDose(P,L,6+M,PL)+1                        ! Increment "ill child" group
                  IF (Employed) DM8HDose(P,L,7+M,PL)                 &
                     = DM8HDose(P,L,7+M,PL)+1                        ! Increment "employed" group
               ENDIF                                                 ! End level exceeded logic 
            ENDDO                                                    ! End loop over dose types
         ENDDO                                                       ! Continue with next table level
         X = MAXVAL(HourSeq(24*I-23:24*I)%FDose)                     ! Find daily max end-of-hour dose
         DailyDose(I)%Max1FDose = X                                  ! Store daily max end-of-hour dose
         EVRModerate = ModEVR1                                       ! Moderate 1-Hour EVR (l/min-m^2)          
         EVRHeavy    = HeavyEVR1                                     ! Heavy 1-Hour EVR (l/min-m^2)     
         DoseM = -1                                                  ! Init max dose at mod EVR
         DoseH = -1                                                  ! Init max dose at heavy EVR
         DO J=1,24                                                   ! Loop over hours in day
            EVR = HourSeq(24*I-23+(J-1))%EVR                         ! Current hourly avg EVR       
            IF (EVR >= EVRModerate .AND. EVR < EVRHeavy)            &        
               DoseM=MAX(HourSeq(24*I-23+(J-1))%FDose, DoseM)        ! Update max at mod EVR
            IF (EVR >= EVRHeavy)                                    &        
               DoseH=MAX(HourSeq(24*I-23+(J-1))%FDose, DoseH)        ! Update max at heavy EVR
         ENDDO                                                       ! End loop over days
         D(1) = X                                                    ! MAX EH Dose
         D(2) = DoseM                                                ! MAX EH Dose during moderate
         D(3) = DoseH                                                ! MAX EH Dose during heavy
         DO L=1,POL(PL)%N_DMEHDoseLev                                ! Loop over levels in DM1HE table
            DO N=0,2                                                 ! Do 3X: all, moderate, heavy    
               M=N*7                                                 ! For indexing tables   
               IF (D(N+1)>=DMEHDoseLev (PL,L)) THEN                  ! If Dose level exceeded
                  DMEHDose(P,L,1+M,PL)=DMEHDose(P,L,1+M,PL)+1        ! Increment "all" group 
                  IF (Child) DMEHDose(P,L,2+M,PL)                    &    
                     = DMEHDose(P,L,2+M,PL)+1                        ! Increment "child" group     
                  IF (Active) DMEHDose(P,L,3+M,PL)                   &
                     = DMEHDose(P,L,3+M,PL)+1                        ! Increment "active" group
                  IF (Active.AND.Child) DMEHDose(P,L,4+M,PL)         &
                     = DMEHDose(P,L,4+M,PL)+1                        ! Increment "active child" group
                  IF (Ill) DMEHDose(P,L,5+M,PL)                      &
                     = DMEHDose(P,L,5+M,PL)+1                        ! Increment "ill" group
                  IF (Ill .AND. Child) DMEHDose(P,L,6+M,PL)          &
                     = DMEHDose(P,L,6+M,PL)+1                        ! Increment "ill child" group 
                  IF (Employed) DMEHDose(P,L,7+M,PL)                 &
                     = DMEHDose(P,L,7+M,PL)+1                        ! Increment "employed" group
               ENDIF                                                 ! End level exceeded logic 
            ENDDO                                                    ! End loop over dose types
         ENDDO                                                       ! Continue with next table level
      ENDIF                                                          ! End hourly logic    
      X        = MAXVAL(HourSeq(24*I-23:24*I)%Dose)                  ! Find daily max ts dose
      DailyDose(I)%MaxTSDose = X                                     ! Store daily max ts dose
      EVRModerate = ModEVRTS                                         ! Moderate ts EVR (l/min-m^2)          
      EVRHeavy    = HeavyEVRTS                                       ! Heavy ts EVR (l/min-m^2)     
      DoseM = -1                                                     ! Init max dose at mod EVR
      DoseH = -1                                                     ! Init max dose at heavy EVR
      DO J=1,TS                                                      ! Loop over hours in day
         EVR = TSSeq(TS*I-(TS-1)+(J-1))%EVR                          ! Current ts avg EVR       
         IF (EVR >= EVRModerate .AND. EVR < EVRHeavy)               &          
            DoseM=MAX(TSSeq(TS*I-(TS-1)+(J-1))%Dose, DoseM)          ! Update max at mod EVR
         IF (EVR >= EVRHeavy)                                       &        
            DoseH=MAX(TSSeq(TS*I-(TS-1)+(J-1))%Dose, DoseH)          ! Update max at heavy EVR
      ENDDO                                                          ! End loop over days    
      D(1) = X                                                       ! MAX TS dose
      D(2) = DoseM                                                   ! MAX TS dose during moderate
      D(3) = DoseH                                                   ! MAX TS dose during heavy
      DO L=1,POL(PL)%N_DMTSDoseLev                                   ! Loop over levels in DMTSD table
         DO N=0,2                                                    ! Do 3X: all, moderate, heavy    
            M=N*7                                                    ! For indexing tables   
            IF (D(N+1)>=DMTSDoseLev(PL,L)) THEN                      ! If dose level exceeded
               DMTSDose(P,L,1+M,PL)=DMTSDose(P,L,1+M,PL)+1           ! Increment "all" group 
               IF (Child) DMTSDose(P,L,2+M,PL)                      &    
                  = DMTSDose(P,L,2+M,PL)+1                           ! Increment "child" group     
               IF (Active) DMTSDose(P,L,3+M,PL)                     &
                  = DMTSDose(P,L,3+M,PL)+1                           ! Increment "active" group      
               IF (Active.AND.Child) DMTSDose(P,L,4+M,PL)           &
                  = DMTSDose(P,L,4+M,PL)+1                           ! Increment "active child" group
               IF (Ill) DMTSDose(P,L,5+M,PL)                        &
                  = DMTSDose(P,L,5+M,PL)+1                           ! Increment "ill" group
               IF (Ill .AND. Child) DMTSDose(P,L,6+M,PL)            &
                  = DMTSDose(P,L,6+M,PL)+1                           ! Increment "ill child" group
               IF (Employed) DMTSDose(P,L,7+M,PL)                   &
                  = DMTSDose(P,L,7+M,PL)+1                           ! Increment "employed" group
            ENDIF                                                    ! End level exceeded logic 
         ENDDO                                                       ! End loop over dose types
      ENDDO                                                          ! Continue with next table level
      X = SUM(TSSeq(TS*I-(TS-1):TS*I)%Dose)/TS                       ! Average daily dose
      DailyDose(I)%AvgDose = X                                       ! Store daily average dose    
      X = SUM(TSSeq(TS*I-(TS-1):TS*I)%IntakeDose)/TS                 ! Average daily intake dose
      DailyDose(I)%IntakeDose = X                                    ! Store average intake dose    
      X = SUM(TSSeq(TS*I-(TS-1):TS*I)%DepDose)                       ! Total daily deposited dose
      DailyDose(I)%IntakeDose = X                                    ! Store deposited dose    
      DO L=1,POL(PL)%N_DAvgDoseLev                                   ! Loop over levels in DAvg table
         IF (X>=DAvgDoseLev(PL,L)) DAvgDose(P,L,PL) =               &
            DAvgDose(P,L,PL)+1                                       ! Increment levels exceeded by X
      ENDDO                                                          ! Continue with next table level
   ENDDO                                                             ! Continue with next day
   EVRModerate = ModEVRTS                                            ! Moderate ts EVR (l/min-m^2)          
   EVRHeavy    = HeavyEVRTS                                          ! Heavy ts EVR (l/min-m^2)     
   DO I=1, NumTS                                                     ! Loop over timesteps
      EVR = TSSeq(I)%EVR                                             ! Current timestep EVR       
      DoseM = -1                                                     ! Reset dose at mod EVR
      DoseH = -1                                                     ! Reset dose at heavy EVR
      X = TSSeq(I)%Dose                                              ! Timestep dose 
      IF (EVR >= EVRModerate .AND. EVR < EVRHeavy)                  &          
         DoseM=TSSeq(I)%Dose                                         ! Update val at mod EVR
      IF (EVR >= EVRHeavy) DoseH=TSSeq(I)%Dose                       ! Update val at heavy EVR
      D(1) = X                                                       ! TS dose
      D(2) = DoseM                                                   ! TS dose during moderate
      D(3) = DoseH                                                   ! TS dose during heavy
      DO L=1,POL(PL)%N_TSDoseLev                                     ! Loop over TSdose table levels
         DO N=0,2                                                    ! Do 3X: all, moderate, heavy    
            M=N*7                                                    ! For indexing tables   
            IF (D(N+1)>=TSDoseLev(PL,L)) THEN                        ! If dose level exceeded
               ALLTSDose(P,L,1+M,PL)=ALLTSDose(P,L,1+M,PL)+1         ! Increment "all" group 
               IF (Child) ALLTSDose(P,L,2+M,PL)                     &    
                  = ALLTSDose(P,L,2+M,PL)+1                          ! Increment "child" group
               IF (Active) ALLTSDose(P,L,3+M,PL)                    &
                  = ALLTSDose(P,L,3+M,PL)+1                          ! Increment "active" group
               IF (Active.AND.Child) ALLTSDose(P,L,4+M,PL)          &
                  = ALLTSDose(P,L,4+M,PL)+1                          ! Increment "active child" group
               IF (Ill) ALLTSDose(P,L,5+M,PL)                       &
                  = ALLTSDose(P,L,5+M,PL)+1                          ! Increment "ill" group
               IF (Ill .AND. Child) ALLTSDose(P,L,6+M,PL)           &
                  = ALLTSDose(P,L,6+M,PL)+1                          ! Increment "ill child" group
               IF (Employed) ALLTSDose(P,L,7+M,PL)                  &
                  = ALLTSDose(P,L,7+M,PL)+1                          ! Increment "employed" group
            ENDIF                                                    ! End level exceeded logic 
         ENDDO                                                       ! End loop over dose types
      ENDDO                                                          ! Continue with next table level  
   ENDDO                                                             ! End loop over timesteps
   X = SUM(DailyDose(1:NumDays)%AvgDose)/NumDays                     ! Average dose over simulation
   DO L=1,POL(PL)%N_SAvgDoseLev                                      ! Loop over SAvgDose table levels
      IF (X>=SAvgDoseLev(PL,L)) SAvgDose(P,L,PL)=SAvgDose(P,L,PL)+1  ! Increment levels exceeded by X
   ENDDO                                                             ! Continue with next table level  
   AvgDose(P,PL)=SUM(TSSeq(:)%Dose)/REAL(NumTS)                      ! Save average doses
   MaxDose(P,PL)=MAXVAL(TSSeq(:)%Dose)                               ! Save max doses
   DO I=1,NumEvents                                                  ! Loop over events
      EventDose(I,PL) = EventSeq(I)%Dose                             ! Save event dose for person
      EventFDose(I,PL)= EventSeq(I)%FDose                            ! Save event fdose for person
      EventInDose(I,PL)= EventSeq(I)%IntakeDose                      ! Save event intake for person
      EventDepDose(I,PL)= EventSeq(I)%DepDose                        ! Save event deposited for person
   ENDDO                                                             ! End loop over events
   IF (DoHours==YES) THEN                                            ! If hours required
      DO I=1,NumHours                                                ! Loop over hours
         HourDose(I,PL) = HourSeq(I)%Dose                            ! Save hour dose for person
         HourIntake(I,PL) = HourSeq(I)%IntakeDose                    ! Save hour intake for person
         HourDep(I,PL) = HourSeq(I)%DepDose                          ! Save hour deposited for person
      ENDDO                                                          ! End loop over events
   ENDIF                                                             ! End hour logic
   DO I=1,NumTS                                                      ! Loop over ts
      TSDose(I,PL) = TSSeq(I)%Dose                                   ! Save ts dose for person
      TSInDose(I,PL) = TSSeq(I)%IntakeDose                           ! Save ts dose for person
      TSDepDose(I,PL) = TSSeq(I)%DepDose                             ! Save ts dose for person         
   ENDDO                                                             ! End loop over ts
   DO I=1,NumDays                                                    ! Loop over days
      DayAvgDose(I,PL) = DailyDose(I)%AvgDose                        ! Save day avg dose for person
      DayMax1Dose(I,PL) = DailyDose(I)%Max1Dose                      ! Save day max1 dose for person
      DayMax8Dose(I,PL) = DailyDose(I)%Max8Dose                      ! Save day max8 dose for person
      DayMaxTSDose(I,PL) = DailyDose(I)%MaxTSDose                    ! Save day ts dose for person
      DayMax1FDose(I,PL) = DailyDose(I)%Max1FDose                    ! Save day max 1-h final dose
      DayIntake(I,PL) = DailyDose(I)%IntakeDose                      ! Save day average intake dose
      DayDep(I,PL)    = DailyDose(I)%DepDose                         ! Save day total deposited dose
   ENDDO                                                             ! End loop over events
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE SummarizeDose                                         ! Return to main APEX program

SUBROUTINE CalcCOHb(P, PL)                                           ! Called from ExposureDose
   INTEGER (KIND=IK), INTENT(IN) :: P, PL                            ! Input variables (person, pollut)
   REAL    (KIND=RK), PARAMETER  :: HLDNE =218.                      ! Haldane constant
   REAL    (KIND=RK), PARAMETER  :: RHBK  =0.32                      ! k factor (eq. E-7)
   REAL    (KIND=RK), PARAMETER  :: CRCMHB=0.995                     ! Correction for MetHb 
   REAL    (KIND=RK) :: COHB, PM47, PICO, PCO2, DIFF, RHB0, B, Dose  ! Local real variables 
   REAL    (KIND=RK) :: KT, K0, K1, D, D0, A1, Z, F1, F2, F3, F4     ! Local real variables
   REAL    (KIND=RK) :: DoseF = 1                                    ! Output to dose units conversion
   INTEGER (KIND=IK) :: Day, I, J, N, DayOneEvents, NPieces          ! Local integer variables
   ! Global constants: HOURSDAY, IK, LogU, LV_MS3, PPM, RK, SL_FM3, ST_MS3
   ! Global vars used: Altitude, COHbFact, DebugLevel, HourSeq, NumEvents, OutUnits, Phys, PPMFact 
   ! Global vars set : EventSeq, ProcLevel, ProcName
   ! Intrinsic procs : EXP, INT, REAL, WRITE
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "CalcCOHb"                                  ! Procedure name for messages 
   IF (DebugLevel>1) WRITE(LogU,SL_FM3) ST_MS3, ProcName(ProcLevel)  ! Message for debugging
   IF (Pol(PL)%OutUnits==UGM3) DoseF = 1.0 / Pol(PL)%PPMFact         ! Factor for units conversion
   PM47 = 760.*EXP(-0.0000386*Altitude)-47.                          ! Pressure of dry gases      
   PCO2 = 0.209*PM47-49.                                             ! O2 Partial pressure in lungs
   RHB0 = 1.39*(Phys(P)%HMGLB*0.01)*CRCMHB                           ! Maximum Hb available (sea level) 
   RHB0 = RHB0*(1.0+(2.76*exp(0.0001429*Altitude))/100.0)            ! Altitude correction for Max Hb
   DayOneEvents = HourSeq(HOURSDAY)%LastEvent                        ! # of repeated events (day one)
   COHB = 0.                                                         ! Initialize blood COHB to zero
   DO I=1-DayOneEvents,NumEvents                                     ! Loop over events
      J = I                                                          ! J is event index 
      IF (J<1) J = J+DayOneEvents                                    ! Repeats the first day events
      Day  = EventSeq(J)%Day                                         ! Day for event J
      PICO = PM47*EventSeq(J)%Exposure*DoseF*1.E-6                   ! Partial pressure of CO 
      DIFF = Phys(P)%Diff + 0.000845*EventSeq(J)%VA - 5.7            ! Adjusted lung diffusivity
      B    = (1./DIFF)+(PM47/EventSeq(J)%VA)                         ! A term used twice, below
      K0   = (PersonDay(Day)%EndGn+PICO/B)/(RHB0*Phys(P)%Blood)      ! CO takeup rate
      K1   = (1.+RHBK*PCO2)/(RHBK*RHB0*HLDNE*B*Phys(P)%Blood)        ! O2 takeup rate
      A1   = K1/(K0+K1)                                              ! Ratio of rate constants
      KT   = (K0+K1)*REAL(EventSeq(J)%Duration)                      ! Scaled event duration 
      Dose = 0.                                                      ! Initialize integral for dose
      Npieces = 1+INT(COHbFact*KT/(1.-COHB/100.)**2)                 ! Number of subevents to use
      DO N=1, Npieces                                                ! Loop over subevents (usually 1)
         D0   = 1.-COHB/100.                                         ! Percent non-COHb at start
         D    = D0-A1                                                ! Dimensionless rate constant
         Z    = KT/(D0*D0*REAL(NPieces))                             ! Dimensionless duration of event
         F1   = 100.*D0*D*Z                                          ! Linear Taylor's series term
         F2   = -F1*A1*Z/2.                                          ! Quadratic Taylor's series term
         F3   = F2*(2.*D-A1)*Z/3.                                    ! Cubic Taylor's series term
         F4   = F2*(A1*A1-8.*A1*D+6.*D*D)*Z*Z/12.                    ! Quartic Taylor's series term
         Dose = Dose + COHB + F1/2. + F2/3. + F3/4. + F4/5.          ! Accumulate average COHB 
         COHB = COHB + F1 + F2 + F3 + F4                             ! COHB at end of subevent
      ENDDO                                                          ! Continue with next subevent
      EventSeq(J)%Dose  = Dose/REAL(NPieces)                         ! Assign mean COHB to result
      EventSeq(J)%FDose = COHB                                       ! Final dose is final COHB value
   ENDDO                                                             ! Continue with next event
   IF (DebugLevel>1) WRITE(LogU,SL_FM3) LV_MS3, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE CalcCOHb                                              ! Return to ExposureDose

SUBROUTINE GeneralDose(P, PL)                                        ! Called from Dose
   INTEGER (KIND=IK), INTENT(IN) :: P, PL                            ! Input variables (person, pollut)
   INTEGER (KIND=IK) ::  I                                           ! Local integer variables
   ! Global vars set : EventSeq, ProcLevel, ProcName
   ! Intrinsic procs : WRITE
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "GeneralDose"                               ! Procedure name for messages 
   IF (DebugLevel>1) WRITE(LogU,SL_FM3) ST_MS3, ProcName(ProcLevel)  ! Message for debugging
   DO I=1,NumEvents                                                  ! Loop over events
      EventSeq(I)%Dose  = EventSeq(I)%Exposure*EventSeq(I)%Ve        ! Assign general dose value
      EventSeq(I)%FDose = EventSeq(I)%Dose                           ! Final dose is same as event dose
   ENDDO                                                             ! Continue with next event
   IF (DebugLevel>1) WRITE(LogU,SL_FM3) LV_MS3, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE GeneralDose                                           ! Return to Dose

END MODULE ExposureDoseModule                                        ! End of source code for module