MODULE SpaceTimeModule                                               ! Compile after the modules below
   ! Latest revisions by MCN on October 17, 2011
   USE GlobalModule                                                  ! Must compile this module first
   USE UtilityModule                                                 ! Must compile this module second
   USE DistributionModule                                            ! Must compile this module third
   IMPLICIT NONE                                                     ! Must declare all variables
   INTEGER (KIND=IK) :: StartJulian = 0                              ! Julian Day of first day of sim
   INTEGER (KIND=IK) :: EndJulian   = 0                              ! Julian Day of last day of sim
   INTEGER (KIND=IK) :: InitSectors=0, InitDistricts=0, InitZones=0  ! Lengths of initial site lists
   INTEGER (KIND=IK) :: RoadDistricts=0                              ! Lengths of initial site lists   
   TYPE (SITE), ALLOCATABLE :: Stemp(:), Dtemp(:), Ztemp(:)          ! Temporary arrays for sites
   TYPE (CDIST),TARGET,ALLOCATABLE :: AQDistr(:,:,:,:)               ! Array of AQ distributions 
CONTAINS

SUBROUTINE CalcDates                                                 ! Called from main APEX program
   INTEGER (KIND=IK) :: I, J, K, YMD                                 ! Local integer variables
   ! Global constants: HOURSDAY, IK, LogU, LV_FMT, LV_MSG, ST_FMT, ST_MSG
   ! Global vars used: DebugLevel, EndDate, StartDate 
   ! Global vars set : Calen, NumDays, NumHours, NumYears, ProcLevel, ProcName
   ! Module vars set : EndJulian, StartJulian
   ! Intrinsic procs : ALLOCATE, MOD, PRINT, WRITE
   ! APEX procedures : CheckAllocation, Fatal, JulianDay, YearMonthDay 
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "CalcDates"                                 ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   StartJulian = JulianDay(StartDate)                                ! Convert start to Julian date
   EndJulian   = JulianDay(EndDate)                                  ! Convert end to Julian date
   PRINT*,       "Simulation start date =", StartDate, StartJulian   ! Message start date to screen
   PRINT*,       "Simulation end date   =", EndDate  , EndJulian     ! Message end date to screen
   WRITE(LogU,*) "Simulation start date =", StartDate, StartJulian   ! Put start date on log
   WRITE(LogU,*) "Simulation end date   =", EndDate  , EndJulian     ! Put end date on log
   NumDays  = 1+EndJulian-StartJulian                                ! # of days in the simulation
   NumHours = HOURSDAY*NumDays                                       ! # of hours in the simulation
   NumTS    = TimestepsPerDay*NumDays                                ! # of ts in the simulation   
   NumYears = 1+EndDate/10000-StartDate/10000                        ! # of years (including partials)
   WRITE(LogU,*) "# Years,  Days        =",NumYears, NumDays         ! Note above values on the Log
   ALLOCATE(Calen(NumDays),STAT=AllocErr)                            ! Space for calendar data
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   Calen(:)%Weekend = 0                                              ! Initialize all days to weekdays
   DO I=1,NumDays                                                    ! Loop over days in simulation
      J   = StartJulian+I-1                                          ! Julian day corresponding to I
      YMD = YearMonthDay(J)                                          ! Convert date to YYYYMMDD form
      K   = MOD(J,7)+1                                               ! K is day of week (1=Sun)
      Calen(I)%Year      = YMD/10000                                 ! Year for each simulation day
      Calen(I)%Month     = MOD(YMD,10000)/100                        ! Month for each simulation day
      Calen(I)%Day       = MOD(YMD,100)                              ! Day for each simulation day
      Calen(I)%DayofWeek = K                                         ! Day of week for each day
      IF ((K==1).OR.(K==7)) Calen(I)%Weekend = 1                     ! 1 is Sunday and 7 is Saturday
      Calen(I)%FirstHour = 1+HOURSDAY*(I-1)                          ! Index of first hour on day I 
      Calen(I)%LastHour  = HOURSDAY*I                                ! Index of last hour on day I
   ENDDO                                                             ! End of loop over days
   IF (DebugLevel>0) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE CalcDates                                             ! Return to main APEX program

INTEGER FUNCTION JulianDay(Date)                                     ! Called within TimeSeriesModule
   INTEGER (KIND=IK), INTENT(IN)  :: Date                            ! Date must be in YYYYMMDD form
   INTEGER (KIND=IK) :: Leap, Z, Month, Day, Year                    ! Temporary variables
   INTEGER (KIND=IK) :: DayMon(12)                                   ! Array for days per month
   ! Global constants: IK, LV_FMT, LV_MSG, ST_FMT, ST_MSG  
   ! Global vars used: DebugLevel
   ! Global vars set : ProcLevel, ProcName
   ! Intrinsic procs : MOD, SUM, WRITE
   ! APEX procedures : Fatal
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "JulianDay"                                 ! Procedure name for messages
   Year        = Date/10000                                          ! YYYY part of the input date
   DayMon = (/31,28,31,30,31,30,31,31,30,31,30,31/)                  ! # days per month (non-leap)
   Leap   = 0                                                        ! Default leap year status (no)
   IF (MOD(Year,4)==0) Leap = 1                                      ! Normal condition for leap year
   IF ((MOD(Year,100)==0).AND.(MOD(Year,400)/=0)) Leap = 0           ! Century rules for leap year
   IF (Leap==1) DayMon(2) = 29                                       ! Extra February day in leap year
   Month = (Date-10000*Year)/100                                     ! MM part of the input date
   IF ((Month<1).OR.(Month>12)) CALL Fatal(2)                        ! Check for valid month
   Day = Date-10000*Year-100*Month                                   ! DD part of input date
   IF ((Day<1).OR.(Day>DayMon(Month))) CALL Fatal(3)                 ! Check for valid day number
   Z = Year-1                                                        ! Formula is based on last year
   JulianDay = Z*365+Z/4-Z/100+Z/400+SUM(DayMon(1:(Month-1)))+Day    ! Formula for Julian day
   IF (DebugLevel>2) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END FUNCTION JulianDay                                               ! Return to calling routine


INTEGER FUNCTION YearMonthDay(JulDay)                                ! Called within TimeSeriesModule
   INTEGER (KIND=IK), INTENT(IN)  :: JulDay                          ! Inverse function to JulianDay()
   INTEGER (KIND=IK) :: Year,Month,Day,N400,N100,N4,N1,Rem,Leap      ! Local integer variables
   INTEGER (KIND=IK) :: DayMon(12)                                   ! Array for days per month
   ! Global constants: IK, LV_FMT, LV_MSG, ST_FMT, ST_MSG  
   ! Global vars used: DebugLevel
   ! Global vars set : ProcLevel, ProcName
   ! Intrinsic procs : MIN, MOD, WRITE
   ! APEX procedures : Fatal
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "YearMonthDay"                              ! Procedure name for messages
   IF (JulDay<1) CALL Fatal(1)                                       ! All JulianDays>0 are valid
   N400 = (JulDay-1)/146097                                          ! 146097 is # days in 400 years
   Rem  = JulDay - (N400*146097)                                     ! Remainder after 400s removed 
   N100 = MIN((Rem-1)/36524,3)                                       ! 36524 days in short century
   Rem  = Rem - (N100*36524)                                         ! Remainder after 100s removed
   N4   = (Rem-1)/1461                                               ! 1461 days in four years
   Rem  = Rem - (N4*1461)                                            ! Remainder after 4s removed
   N1   = MIN((Rem-1)/365,3)                                         ! 365 days in short year
   Rem  = Rem - (N1*365)                                             ! Remainder after years removed
   Year = (N400*400) + (N100*100) + (N4*4) + N1 + 1                  ! Current year number
   DayMon = (/31,28,31,30,31,30,31,31,30,31,30,31/)                  ! # days per month (non-leap)  
   Leap = 0                                                          ! Default leap yesr status (no)
   IF (MOD(Year,4)==0) Leap=1                                        ! Normal leap year condition
   IF ((MOD(Year,100)==0).AND.(MOD(Year,400)/=0)) Leap=0             ! Century leap year condition
   IF (Leap==1) DayMon(2)=29                                         ! # February days in leap year
   DO Month=1,12                                                     ! Loop over months 
      IF (Rem<=DayMon(Month)) EXIT                                   ! Exit if rem <= days in month
      Rem = Rem-DayMon(Month)                                        ! If not, then decrease rem
   ENDDO                                                             ! Try the next month
   Day = Rem                                                         ! Day number within current month
   YearMonthDay = (Year*10000) + (Month*100) + Day                   ! Put pieces together
   IF (DebugLevel>2) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END FUNCTION YearMonthDay                                            ! Return to calling routine


SUBROUTINE ReadSiteLists                                             ! Called from main APEX program
   INTEGER   (KIND=IK)  :: S, D, Z, MonStart, MonEnd, LType, K, N    ! Local integer variables
   REAL      (KIND=RK)  :: Dist, Road                                ! Real variables for distances
   CHARACTER (LEN=LEN4) :: String                                    ! Buffer for input strings
   TYPE      (Site)     :: Buf                                       ! Buffer for site parameters
   ! Global constants: IK, LEN3, LogU, LV_FMT, LV_MSG, MonU, NO, RK, SectU, ST_FMT, ST_MSG, YES, ZoneU
   ! Global vars used: AirRadius, Center, CityRadius, CList, CountyList, DebugLevel, 
   !                   LogDistrict, LogSectors, LogZones, TractList, ZoneRadius
   ! Global vars set : Ios, ProcLevel, ProcName
   ! Module vars used: EndJulian, StartJulian
   ! Module vars set : Dtemp, InitDistricts, InitSectors, InitZones, Stemp, Ztemp
   ! Intrinsic procs : ALLOCATE, ANY, CLOSE, READ, REWIND, WRITE
   ! APEX procedures : CheckAllocation, Distance, Fatal, GetLine, JulianDay, OpenInput
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "ReadSiteLists"                             ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging   
   ! First get counts of sectors, districts and zones within range.
   InitSectors = 0                                                   ! Initialize count of sectors
   CALL OpenInput(SectU,h=NO)                                        ! Open the sectors list file
   DO                                                                ! Loop over records on file
      CALL GetLine(SectU,String,LType)                               ! Get one record
      IF (Ios(SectU)<0) EXIT                                         ! Exit loop on end-of-file
      READ (String,*,IOSTAT=Ios(SectU)) Buf%Name, Buf%Lat, Buf%Lon   ! Input sector name and location
      Dist = Distance(Center,Buf)                                    ! Find distance to study center
      IF (Dist>CityRadius) CYCLE                                     ! Reject sector if too far away
      IF (CountyList==YES .AND. TractList==YES) THEN                 ! If both lists defined
         IF (.NOT. ANY(CList(:)==Buf%Name(1:5)).AND..NOT.           &        
            ANY(TList(:)==Buf%Name(1:11))) CYCLE                     ! Drop if county/tract not on list 
      ELSEIF (CountyList==YES) THEN                                  ! If only county list defined       
         IF (.NOT. ANY(CList(:)==Buf%Name(1:5)))  CYCLE              ! Drop if county not on list 
      ELSEIF (TractList==YES)  THEN                                  ! If only tract list defined
         IF (.NOT. ANY(TList(:)==Buf%Name(1:11))) CYCLE              ! Drop if tract not on list 
      ENDIF                                                          ! End of county/tract list logic
      InitSectors = InitSectors+1                                    ! Sector is acceptable  
   ENDDO                                                             ! Continue with next input record
   REWIND(SectU)                                                     ! Rewind sector file to beginning
   IF (InitSectors==0) CALL Fatal(1)                                 ! Stop run if no sectors found
   CALL OpenInput(MonU,preblank=YES,h=NO)                            ! Open air districts list file
   InitDistricts = 0                                                 ! Initialize district counter
   RoadDistricts = 0                                                 ! Initialize road district counter
   DO                                                                ! Loop over input records
      CALL GetLine(MonU,String,LType)                                ! Get one record from input file
      IF (Ios(MonU)/=0) EXIT                                         ! Exit loop on end-of-file
      IF (AirRadius/=99999) THEN                                     ! If AirRadius is defined...
         READ(String,*,IOSTAT=Ios(MonU)) Buf%Name,Buf%Lat,Buf%Lon,  &    
            MonStart, MonEnd                                         ! Extract site info from record
         Buf%Radius = AirRadius                                      ! Setting array radius
      ELSE                                                           ! If AirRadius is not defined...
         READ(String,*,IOSTAT=Ios(MonU)) Buf%Name,Buf%Lat,Buf%Lon,  &    
            MonStart, MonEnd, Buf%Radius                             ! Extract site info from record       
         IF (Buf%Radius < 0) CALL Fatal(5)                           ! Error if radii < 0 
      ENDIF                                                          ! End AirRadius/file read logic
      IF (JulianDay(MonStart)>StartJulian) CYCLE                     ! Reject site if data starts late
      IF (JulianDay(MonEnd)  <EndJulian)   CYCLE                     ! Reject site if data stops early 
      Dist = Distance(Center,Buf)                                    ! Find distance to study center
      IF(Dist>(CityRadius+Buf%Radius)) CYCLE                         ! If site within range then cycle
      InitDistricts = InitDistricts+1                                ! District is acceptable
      IF (INDEX(Up(Buf%Name),"ROAD")  >0) THEN                       ! If this is a roadway district 
         RoadDistricts = RoadDistricts+1                             ! Count number of roadway districts
      ENDIF                                                          ! End conditional
   ENDDO                                                             ! Continue with next record
   IF (RoadDistricts==0 .AND. RoadWayFlag==YES) CALL Fatal(4)        ! If no roadway districts are found
   REWIND(MonU)                                                      ! Rewind district file to start
   IF (InitDistricts==0) CALL Fatal(2)                               ! Stop run if no districts found
   CALL OpenInput(ZoneU,preblank=YES,h=NO)                           ! Open temperature zone list file
   InitZones = 0                                                     ! Initialize zone counter
   DO                                                                ! Loop over records on input file
      CALL GetLine(ZoneU,String,LType)                               ! Get one record from input file
      IF (Ios(ZoneU)/=0) EXIT                                        ! Exit loop on end-of-file
      READ(String,*,IOSTAT=Ios(ZoneU)) Buf%Name,Buf%Lat,Buf%Lon,    &
         MonStart, MonEnd                                            ! Extract site info from record
      IF (JulianDay(MonStart)>StartJulian) CYCLE                     ! Reject site if data starts late
      IF (JulianDay(MonEnd)  <EndJulian)   CYCLE                     ! Reject site if data stops early 
      Dist = Distance(Center,Buf)                                    ! Find distance to study center
      IF(Dist>(CityRadius+ZoneRadius)) CYCLE                         ! If site within range then...
      InitZones = InitZones+1                                        ! Zone is acceptable
   ENDDO                                                             ! Continue with next record
   REWIND(ZoneU)                                                     ! Rewind zone file to beginning
   IF (InitZones==0) CALL Fatal(3)                                   ! Stop run if no zones found
   ALLOCATE (STemp(InitSectors),STAT=AllocErr)                       ! Allocate space for sector list
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (DTemp(InitDistricts),STAT=AllocErr)                     ! Allocate space for district list
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (ZTemp(InitZones),STAT=AllocErr)                         ! Allocate space for zone list
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (NearbyS(InitSectors,0:InitSectors),STAT=AllocErr)       ! Allocate nearby sectors array
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc   
   ! Read input files again, now that space is allocated for the data
   S = 0                                                             ! Position on initial list
   Buf%Index = 0                                                     ! Position on input file
   DO                                                                ! Loop over records on file
      CALL GetLine(SectU,String,LType)                               ! Get one record
      IF (Ios(SectU)<0) EXIT                                         ! Exit loop on end-of-file
      READ (String,*,IOSTAT=Ios(SectU)) Buf%Name, Buf%Lat, Buf%Lon   ! Input sector name and location
      Buf%Index = Buf%Index+1                                        ! Set index to position on file
      Buf%DistC = Distance(Center,Buf)                               ! Find distance to study center
      IF (Buf%DistC>CityRadius) CYCLE                                ! Reject sector if too far away
      IF (CountyList==YES .AND. TractList==YES) THEN                 ! If both lists defined
         IF (.NOT. ANY(CList(:)==Buf%Name(1:5)).AND..NOT.           &        
            ANY(TList(:)==Buf%Name(1:11))) CYCLE                     ! Drop if county/tract not on list 
      ELSEIF (CountyList==YES) THEN                                  ! If only county list defined       
         IF (.NOT. ANY(CList(:)==Buf%Name(1:5)))  CYCLE              ! Drop if county not on list 
      ELSEIF (TractList==YES)  THEN                                  ! If only tract list defined
         IF (.NOT. ANY(TList(:)==Buf%Name(1:11))) CYCLE              ! Drop if tract not on list 
      ENDIF                                                          ! End of county/tract list logic
      S         = S+1                                                ! Sector is acceptable 
      STemp(S)  = Buf                                                ! Copy to site list 
   ENDDO                                                             ! Continue with next input record
   CLOSE(SectU)                                                      ! Close sector list input file     
   NearbyS(:,:) = -999                                               ! Initializing NearbyS Array
   NearbyS(:,0) = 0                                                  ! Initializing count NearbyS
   IF (LogSectors==YES) THEN                                         ! If user wants sector info on log
      WRITE(LogU,*) 'Preliminary list of sectors'                    ! Header for table on log file
      DO S=1,InitSectors                                             ! Loop over initial sector list
         WRITE(LogU,'(I6,A1,A20,F15.5,F15.5)')                      &
            S," ",Stemp(S)%Name,Stemp(S)%Lat,Stemp(S)%Lon            ! Put sector info on log file
      ENDDO                                                          ! Continue with next sector
   ENDIF                                                             ! End of sector info logic
   D = 0                                                             ! Initialize district counter
   DO                                                                ! Loop over input records
      CALL GetLine(MonU,String,LType)                                ! Get one record from input file
      IF (Ios(MonU)/=0) EXIT                                         ! Exit loop on end-of-file
      IF (AirRadius/=99999) THEN                                     ! If AirRadius is defined...
         READ(String,*,IOSTAT=Ios(MonU)) Buf%Name,Buf%Lat,Buf%Lon,     &    
            MonStart, MonEnd                                         ! Extract site info from record
         Buf%Radius = AirRadius                                      ! Setting district radius
      ELSE                                                           ! If AirRadius is not defined...
         READ(String,*,IOSTAT=Ios(MonU)) Buf%Name,Buf%Lat,Buf%Lon,     &    
            MonStart, MonEnd, Buf%Radius                             ! Extract site info from record      
      ENDIF                                                          ! End AirRadius/file read logic
      IF (JulianDay(MonStart)>StartJulian) CYCLE                     ! Reject site if data starts late
      IF (JulianDay(MonEnd)  <EndJulian)   CYCLE                     ! Reject site if data stops early 
      Buf%DistC = Distance(Center,Buf)                               ! Find distance to study center
      IF(Buf%DistC>(CityRadius+Buf%Radius)) CYCLE                    ! Reject if out of range
      D         = D+1                                                ! Increment district counter
      DTemp(D)  = Buf                                                ! Copy to site list
   ENDDO                                                             ! Continue with next record
   CLOSE(MonU)                                                       ! Close the district list file
   IF (LogDistrict==YES) THEN                                        ! If user wants district info
      WRITE(LogU,*) 'Preliminary list of districts'                  ! Put header on log file table 
      DO D=1,InitDistricts                                           ! Loop over initial district list
         WRITE(LogU,'(I6,A1,A20,F15.5,F15.5,F15.5)')                &       
            D," ",Dtemp(D)%Name,Dtemp(D)%Lat,Dtemp(D)%Lon,          &
               Dtemp(D)%Radius                                       ! Put district info on log
      ENDDO                                                          ! Continue with next district
   ENDIF                                                             ! End of district info logic 
   Z = 0                                                             ! Initialize zone counter
   DO                                                                ! Loop over records on input file
      CALL GetLine(ZoneU,String,LType)                               ! Get one record from input file
      IF (Ios(ZoneU)/=0) EXIT                                        ! Exit loop on end-of-file
      READ(String,*,IOSTAT=Ios(ZoneU)) Buf%Name,Buf%Lat,Buf%Lon,    &
         MonStart, MonEnd                                            ! Extract site info from record
      IF (JulianDay(MonStart)>StartJulian) CYCLE                     ! Reject site if data starts late
      IF (JulianDay(MonEnd)  <EndJulian)   CYCLE                     ! Reject site if data stops early 
      Buf%DistC = Distance(Center,Buf)                               ! Find distance to study center
      IF(Buf%DistC>(CityRadius+ZoneRadius)) CYCLE                    ! Reject if out of range
      Z         = Z+1                                                ! Increment zone counter
      ZTemp(Z)  = Buf                                                ! Copy to site list
   ENDDO                                                             ! Continue with next record
   CLOSE(ZoneU)                                                      ! Close the zone list file
   IF (LogZones==YES) THEN                                           ! If user wants zone info on log
      WRITE(LogU,*) ' Preliminary list of zones'                     ! Put header on log file table
      DO Z=1,InitZones                                               ! Loop over initial zone list 
         WRITE(LogU,'(I6,A1,A20,F15.5,F15.5)')                      &    
            Z," ",Ztemp(Z)%Name,Ztemp(Z)%Lat,Ztemp(Z)%Lon            ! Put zone info on log  
      ENDDO                                                          ! Continue with next zone
   ENDIF                                                             ! End of zone info logic
   WRITE(LogU,*)                                                     ! Put blank line on log
   IF (DebugLevel>0) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE ReadSiteLists                                         ! Return to main APEX program


SUBROUTINE MatchSites                                                ! Called from main APEX program
   INTEGER (KIND=IK) :: S, D, Z, SF, DF, ZF, N, K, PL                ! Local integer variables 
   REAL    (KIND=RK) :: Dist, MaxAirRadius                           ! Local variable for distance (km) 
   ! Global constants: IK, LogU, LV_FMT, LV_MSG, RK, SiteU, ST_FMT, ST_MSG, YES
   ! Global vars used: DebugLevel, LogDistrict, LogSectors, LogZones, NumDays, NumHours, 
   !                   ZoneRadius,
   ! Global vars set : AQData, DList, NumDistricts, NumSectors, NumZones, 
   !                   ProcLevel, ProcName, SList, TemData, ZList
   ! Module vars used: DTemp, InitDistricts, InitSectors, InitZones, Stemp, Ztemp
   ! Intrinsic procs : ALLOCATE, ANY, CLOSE, DEALLOCATE, TRIM, WRITE
   ! APEX procedures : CheckAllocation, Distance
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "MatchSites"                                ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging   
   SF = 0                                                            ! Initialize valid sector counter
   Stemp(:)%NumS = 0                                                 ! Start with null mapping
   IF (RoadLastFlag==YES .AND. RoadWayFlag==NO) THEN                 ! If incompatible...
      CALL Warning(1)                                                ! Call Warning 
      RoadLastFlag=NO                                                ! Toggle flag
      DO PL=1,NumPollutants                                          ! Loop over all pollutants
         Pol(PL)%RoadLast=NO                                         ! Set flag back to NO
      ENDDO                                                          ! End pollutant loop
   ENDIF                                                             ! End roadway logic
   MaxAirRadius = MAXVAL(Dtemp(:)%Radius)                            ! Start with maximum air radius
   DO S=1,InitSectors                                                ! Loop over sector list
      ! Find nearest district
      Stemp(S)%DistD = MaxAirRadius                                  ! Start with maximum allowed
      Stemp(S)%RoadD = CityRadius + MaxAirRadius                     ! Start with maximum allowed
      Stemp(S)%NumD  = 0                                             ! Start with no selected district
      Stemp(S)%NumRD = 0                                             ! Start with no selected road dist
      DO D=1,InitDistricts                                           ! Loop over district list
         Dist = Distance(Stemp(S),Dtemp(D))                          ! Distance from S to D in km               
         IF (INDEX(Up(Dtemp(D)%Name),"ROAD")>0) THEN                 ! If this is a roadway district          
            IF (Dist<Stemp(S)%RoadD) THEN                            ! If this is the closest so far...
                Stemp(S)%RoadD = Dist                                ! Update nearest distance
                Stemp(S)%NumRD = D                                   ! Indicate nearest district #
            ENDIF                                                    ! End of new-nearest logic
         ELSE                                                        ! If not a roadway district
            IF (Dist<Stemp(S)%DistD) THEN                            ! If this is the closest so far...
                Stemp(S)%DistD = Dist                                ! Update nearest distance
                Stemp(S)%NumD  = D                                   ! Indicate nearest district #
            ENDIF                                                    ! End of regular AQ logic
         ENDIF                                                       ! End of new-nearest logic     
      ENDDO                                                          ! Continue with next district     
      ! Now find nearest zone
      Stemp(S)%DistZ = ZoneRadius                                    ! Start with maximum allowed
      Stemp(S)%NumZ  = 0                                             ! Start with no selected zone
      DO Z=1,InitZones                                               ! Loop over initial zone list
         Dist = Distance(Stemp(S),Ztemp(Z))                          ! Distance from S to Z in km
         IF (Dist<Stemp(S)%DistZ) THEN                               ! If this is the closest so far...
            Stemp(S)%DistZ = Dist                                    ! Update nearest distance
            Stemp(S)%NumZ  = Z                                       ! Indicate nearest zone #
         ENDIF                                                       ! End of new-nearest logic
      ENDDO                                                          ! Continue with next zone
      IF (Stemp(S)%NumD>0 .AND. Stemp(S)%NumZ>0) THEN                ! If sector has district and zone 
         SF = SF+1                                                   ! Increment valid sector counter
         Stemp(S)%NumS = SF                                          ! Map old to new sector numbers
      ENDIF                                                          ! End of block for valid sectors
   ENDDO                                                             ! Continue with next sector
   NumSectors = SF                                                   ! Count of valid sectors
   ALLOCATE(SList(0:NumSectors),STAT=AllocErr)                       ! Make space for valid sectors
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   DO S=1,InitSectors                                                ! Loop over initial sector list  
      IF (Stemp(S)%NumS>0) SList(Stemp(S)%NumS) = STemp(S)           ! Copy valid sector to new pos.
   ENDDO                                                             ! Continue with next sector  
   SList(0)%NumD = -1                                                ! Sector zero uses district -1 
   SList(0)%NumRD = -1                                               ! Sector zero uses district -1
   DF = 0                                                            ! Initialize valid district count
   Dtemp(:)%Index = 0                                                ! Index holds old->new mapping  
   DO D=1,InitDistricts                                              ! Loop over initial district list
      IF (ANY(SList(:)%NumD==D) .OR. ANY(SList(:)%NumRD==D)) THEN    ! If any sector uses this district 
         DF = DF+1                                                   ! Then increment count on new list
         Dtemp(D)%Index = DF                                         ! Old district D will become DF     
      ENDIF                                                          ! End of district mapping logic
   ENDDO                                                             ! Continue with next district
   NumDistricts = DF                                                 ! Set global variable 
   ALLOCATE(DList(0:NumDistricts),STAT=AllocErr)                     ! Make space for district array
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   DO D=1,InitDistricts                                              ! Loop over initial district list
      IF (Dtemp(D)%Index>0) DList(Dtemp(D)%Index) = Dtemp(D)         ! Copy data to new position
   ENDDO                                                             ! Continue with next district
   ZF = 0                                                            ! Initialize valid zone counter
   Ztemp(:)%Index = 0                                                ! Index holds old->new mapping
   DO Z=1,InitZones                                                  ! Loop over initial zone list
      IF (ANY(SList(:)%NumZ==Z)) THEN                                ! If any sector uses this zone  
         ZF = ZF+1                                                   ! Then increment count on new list
         Ztemp(Z)%Index = ZF                                         ! Old zone Z will become zone ZF
      ENDIF                                                          ! End of zone mapping logic
   ENDDO                                                             ! Continue with next zone
   NumZones = ZF                                                     ! Get global zone count variable
   ALLOCATE(ZList(0:NumZones),STAT=AllocErr)                         ! Make space for final zone list
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   DO Z=1,InitZones                                                  ! Loop over initial zone list
      IF (Ztemp(Z)%Index>0) ZList(Ztemp(Z)%Index) = Ztemp(Z)         ! Copy data to new position
   ENDDO                                                             ! Continue with next zone
   DO S=1,NumSectors                                                 ! Loop over final sector list
      SList(S)%NumD = Dtemp(SList(S)%NumD)%Index                     ! Update district numbering
      IF (RoadWayFlag==YES) THEN                                     ! If roadway flag yes
         SList(S)%NumRD = Dtemp(SList(S)%NumRD)%Index                ! Update roadway district numbering      
      ELSE                                                           ! If no
         SList(S)%NumRD = 0                                          ! Set to zero
      ENDIF                                                          ! End roadway logic
      SList(S)%NumZ = Ztemp(SList(S)%NumZ)%Index                     ! Update zone numbering
   ENDDO                                                             ! Continue with next sector
   DO S=1,NumSectors                                                 ! Looping over all sectors
      N = 1                                                          ! Initializing count
      DO K=1,NumSectors                                              ! Looping over all sectors
         IF (Distance(SList(S),SList(K)) <= NearbyRadius) THEN       ! If distance is less than radius
            NearbyS(S,N) = K                                         ! Keep track of close tracts
            NearbyS(S,0) = NearbyS(S,0) + 1                          ! Increment counter
            N = N + 1                                                ! Increment N
         ENDIF                                                       ! End nearby logic
      ENDDO                                                          ! End loop over sectors
   ENDDO                                                             ! End loop over sectors   
   WRITE(LogU,*)                                                     ! Put blank line on log file 
   WRITE(LogU,*) " Final number of sectors   ", NumSectors           ! Put sector count on log
   WRITE(LogU,*) " Final number of districts ", NumDistricts         ! Put district count on log
   WRITE(LogU,*) " Final number of zones     ", NumZones             ! Put zone count on log
   DEALLOCATE(Stemp,Dtemp,Ztemp)                                     ! Free memory for temporary arrays
   WRITE(SiteU,'(A240)') " sector#    latitude   longitude   secto" &
      // "rname                                air# airdista"       &
      // "nce airlatitude airlongitude airname              "       &
      // "                      met# metdistance metlatitude"       &
      // " metlongitude metname                             "        ! Header for site list file
   DO S=1,NumSectors                                                 ! Loop over sectors
      D = Slist(S)%NumD                                              ! District number for sector I
      Z = SList(S)%NumZ                                              ! Zone number for sector I
      WRITE(SiteU,'(I8,2F12.4,2X,A40,2(I7,3F12.4,2X,A40))')         &
         S,SList(S)%Lat,SList(S)%Lon,SList(S)%Name,                 &
         D,SList(S)%DistD,DList(D)%Lat,DList(D)%Lon,                &
         DList(D)%Name,Z,SList(S)%DistZ,ZList(Z)%Lat,               &
         ZList(Z)%Lon,ZList(Z)%Name                                  ! Write sector info to log file
   ENDDO                                                             ! Continue with next sector
   CLOSE(SiteU)                                                      ! Close site list file
   IF (LogSectors==YES) THEN                                         ! If user wants sectors on log  
      WRITE(LogU,'(/A33)') ' Final list of population sectors'       ! Put sector list on log file
      WRITE(LogU,*) "sector# latitude longitude distr# "            &
         // "distance zone# distance  sector name"                   ! Column identifiers for table
      DO S=1,NumSectors                                              ! Loop over sectors
         WRITE(LogU,'(I7,2F10.3,I6,F10.3,I5,F10.3,2X,A30)')         &
          S,SList(S)%Lat,SList(S)%Lon,Slist(S)%NumD,Slist(S)%DistD, &
          SList(S)%NumZ,SList(S)%DistZ,SList(S)%Name(1:30)           ! Write sector info to log file
      ENDDO                                                          ! Continue with next sector
   ENDIF                                                             ! End of logic for sector listing
   IF (LogDistrict==YES) THEN                                        ! If user wants districts on log
      WRITE(LogU,'(/A36)') ' Final list of air quality districts'    ! Put district list on log file
      DO D=1,NumDistricts                                            ! Loop over districts
        WRITE(LogU,*)D,DList(D)%Lat,DList(D)%Lon,                   &
          TRIM(DList(D)%Name),DList(D)%Radius                        ! Write district info to log file
      ENDDO                                                          ! Continue with next district
   ENDIF                                                             ! End of logic for district list 
   IF (LogZones==YES) THEN                                           ! If user wants zones on log
      WRITE(LogU,'(/A32)') ' Final list of meteorology zones'        ! Put zone list on log file
      DO Z=1,NumZones                                                ! Loop over zones
        WRITE(LogU,*)Z,Zlist(Z)%Lat,ZList(Z)%Lon,TRIM(ZList(Z)%Name) ! Write zone info to log file
      ENDDO                                                          ! Continue with next zone
      WRITE(LogU,*)                                                  ! Put blank line on log file
   ENDIF                                                             ! End of logic for zone list
   ALLOCATE (AQData(NumPollutants,(-1*TimestepsPerDay+1):NumTS,     &
      -1:NumDistricts),STAT=AllocErr)                                ! Make space for air data
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (TemData(NumDays,NumZones),STAT=AllocErr)                ! Make space for temperature data
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (HourMtr(NumHours,NumZones),STAT=AllocErr)               ! Make space for hourly met. data
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   IF (DebugLevel>0) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE MatchSites                                            ! Return to main APEX program

    
REAL FUNCTION Distance(P1,P2)                                        ! Called from Select Sites 
   TYPE(Site),   INTENT(IN) :: P1,P2                                 ! Input site variables      
   REAL (KIND=8), PARAMETER :: E2 = 0.00672265                       ! Square of earth's eccentricity           
   REAL (KIND=8), PARAMETER :: A  = 6378.17                          ! Semimajor axis length 
   REAL (KIND=8), PARAMETER :: F  = 0.01745329252                    ! Radians in one degree
   REAL (KIND=8) :: T1, T2, Z, AC, R                                 ! Local real variables
   ! Intrinsic procs : ACOS, COS, SIN, SQRT
   ! This function is quite good even at the poles
   T1 = ATAN(TAN(F*REAL(P1%Lat,8))*(1-E2))                           ! Radl vect2 angle at Earth cntr
   T2 = ATAN(TAN(F*REAL(P2%Lat,8))*(1-E2))                           ! Radl vect2 angle at Earth cntr
   Z  = COS(F*(REAL(P1%Lon,8)-REAL(P2%Lon,8)))                       ! Cosine of longitude differnce
   AC = ACOS(COS(T1)*COS(T2)*Z+SIN(T1)*SIN(T2))                      ! Net angle btwn radial vectors
   R  = SQRT((1.-E2)/(1.-E2*COS((T1+T2)/2.)**2))                     ! Earth's radius at avg latitude   
   Distance = A*R*AC                                                 ! Distance between the points
END FUNCTION Distance                                                ! Return to calling routine


SUBROUTINE ReadMeteorologyData                                       ! Called from main APEX program
   CHARACTER(LEN=LEN4) :: String                                     ! Buffer for input record
   CHARACTER(LEN=LEN2) :: N, SubS                                    ! Buffer for dist and site names
   INTEGER (KIND=IK)   :: Date, I=0, J, K, LType, NMiss, TotMiss=0   ! Local integer variables
   INTEGER (KIND=IK)   :: Hour, H, PrevHour, PrevDay,HoursPerTS      ! Local integer variables
   REAL    (KIND=RK)   :: Temp, Humid, Wind, Dir                     ! Buffer in case inputs are real
   CHARACTER(LEN=2)    :: Precip                                     ! Precipitation code
   CHARACTER(LEN=LEN1), POINTER :: PR(:)                             ! Pointer for precipitation
   INTEGER (KIND=IK), POINTER :: MT(:), AT(:), WD(:), HV(:)          ! Pointers for distrib arguments
   INTEGER (KIND=RK),   ALLOCATABLE   :: Buffer(:)                   ! Buffer for met data 
   ! Global constants: IK, LEN2, LEN3, LogU, LV_FMT, LV_MSG, RK, ST_FMT, ST_MSG, TempU
   ! Global vars used: DebugLevel, NumDays, NumZones
   ! Global vars set : Ios, ProcLevel, ProcName, TemData  
   ! Module vars used: EndJulian, StartJulian
   ! Intrinsic procs : ADJUSTL, CLOSE, COUNT, INDEX, MINVAL, NINT, READ, TRIM, WRITE
   ! APEX procedures : CountNums, Distrib, Fatal, GetLine, OpenInput, Up
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "ReadMeteorologyData"                       ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging 
   HourMtr(:,:)%Temp     = -999                                      ! Flag for missing mtr data
   TemData(:,:)%MaxTemp  = -999                                      ! Flag for missing max temp 
   TemData(:,:)%AvgTemp  = -999                                      ! Flag for missing avg temp 
   TemData(:,:)%TempCatM = 0                                         ! Initialize categories to zero
   TemData(:,:)%TempCatA = 0                                         ! Initialize categories to zero
   CALL OpenInput(TempU,h=NO)                                        ! Open temperature file for input
   DO                                                                ! Loop over records on input file
      CALL GetLine(TempU,String,LType)                               ! Get one record from input file
      IF (Ios(TempU)<0) EXIT                                         ! If end-of-file, exit DO loop
      IF (INDEX(Up(String),"NAME")==1) THEN                          ! If keyword is NAME, process it
         SubS = TRIM(ADJUSTL(String(LType+1:)))                      ! Get part on right of '='
         IF (SubS=="") CALL Fatal(1)                                 ! If site has no name, then error
         I = MINVAL(ZList%Index,MASK=(ZList%Name==SubS))             ! Find name in Zlist names
         PrevHour=0                                                  ! Initialize value of previous hour
         PrevDay =0                                                  ! Initialize value of previous day
      ENDIF                                                          ! I is the zone number 
      IF (I<=0) CYCLE                                                ! Skip line if zone not valid
      IF (I>NumZones) CYCLE                                          ! This means SubS is not on ZList
      IF (LType>=0) CYCLE                                            ! Only numeric data below here
      K = CountWords(String)                                         ! Count # items on input line
      SELECT CASE (K)                                                ! Select based on # values   
      CASE (3)                                                       ! If only one value    
      READ (String,*,IOSTAT=Ios(TempU)) Date, Hour, Temp             ! Get meteorological data     
      CASE (4)                                                       ! If two values   
      READ (String,*,IOSTAT=Ios(TempU)) Date, Hour, Temp, Humid      ! Get meteorological data     
      CASE (5)                                                       ! If three values
      READ (String,*,IOSTAT=Ios(TempU)) Date, Hour, Temp, Humid,    & 
         Precip                                                      ! Get meteorological data     
      CASE (6)                                                       ! If 4 values
      READ (String,*,IOSTAT=Ios(TempU)) Date, Hour, Temp, Humid,    &
         Precip, Wind                                                ! Get meteorological data     
      CASE (7)                                                       ! If 5 values
      READ (String,*,IOSTAT=Ios(TempU)) Date, Hour, Temp, Humid,    &
         Precip, Wind, Dir                                           ! Get meteorological data     
      END SELECT                                                     ! End select # words on line
      IF (Ios(TempU)/=0) CALL Fatal(2)                               ! Data not numeric
      J = 1 + JulianDay(Date) - StartJulian                          ! Day number for this record 
      IF (J<1.OR.J>NumDays) CYCLE                                    ! Drop dates out of range
      IF (Hour<1.OR.Hour>24) THEN                                    ! If hours out of range
         WRITE(LogU,*) "Hours out of range in meteorology data ",   &
            "in Zone ", TRIM(ZList(I)%Name), " on day", Date         ! Write error message to log
        CALL Fatal(3)                                                ! Exit run
      ENDIF                                                          ! End hour logic
      H = (J-1)*24 + Hour                                            ! Hour number for this record
       IF (H/=PrevHour+1) THEN                                       ! If hours not sequential           
         WRITE(LogU,*) "Hours or days out of sequence in Zone ",    &
            TRIM(ZList(I)%Name), " on day", Date, " hour", Hour      ! Write warning message to log
         CALL Warning(1)                                             ! Write warning
      ENDIF                                                          ! End hour order logic 
      HourMtr(H,I)%Temp    = INT(Temp)                               ! Set hourly temp
      HourMtr(H,I)%Humid   = INT(Humid)                              ! Set hourly humidity
      HourMtr(H,I)%Precip  = Precip                                  ! Set hourly precip
      HourMtr(H,I)%Wind    = INT(Wind)                               ! Set hourly windspeed   
      HourMtr(H,I)%Dir     = INT(Dir)                                ! Set hourly wind direction
       IF (Hour==24) THEN                                            ! If end of day                                       
         TemData(J,I)%AvgTemp = INT(SUM(HourMtr(H-23:H,I)%Temp)/24)  ! Avg temp for day
         TemData(J,I)%MaxTemp = MAXVAL(HourMtr(H-23:H,I)%Temp)       ! Max temp for day
       ENDIF                                                         ! End end-of-day logic 
      PrevHour=H                                                     ! Set value of previous hour
      PrevDay =J                                                     ! Set value of previous day
   ENDDO                                                             ! Continue loop over input records
   CLOSE(TempU)                                                      ! Close the temperature file
   IF (TimeStepsPerDay<24) THEN                                      ! If timestep more than 1 hr
      HoursPerTS = 24/TimeStepsPerDay                                ! Number of hours per step      
      ALLOCATE(Buffer(1:HoursPerTS))                                 ! Allocate met data buffer  
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc
      DO I=1, NumZones                                               ! Loop over zones 
         DO H=1, NumHours-HoursPerTS-1, HoursPerTS                   ! Loop over hours
            Buffer(1:HoursPerTS) =                                  &
               HourMtr(H:(H+HoursPerTS-1),I)%Temp                    ! Copy temp data for timestep 
            HourMtr(H:H+HoursPerTS-1,I)%Temp =                      &   
               INT(SUM(Buffer(1:HoursPerTS))/HoursPerTS)             ! Average temperature for ts
            Buffer(1:HoursPerTS) =                                  &
               HourMtr(H:H+HoursPerTS-1,I)%Humid                     ! Copy humidity data for timestep 
            HourMtr(H:H+HoursPerTS-1,I)%Humid =                     &   
               INT(SUM(Buffer(1:HoursPerTS))/HoursPerTS)             ! Average humidity for ts
            Precip = HourMtr(H,I)%Precip                             ! Copy precip data for timestep 
            HourMtr(H:H+HoursPerTS-1,I)%Precip = Precip              ! Total precip for ts
            Buffer(1:HoursPerTS) =                                  &
               HourMtr(H:H+HoursPerTS-1,I)%Wind                      ! Copy wndspd data for timestep 
            HourMtr(H:H+HoursPerTS-1,I)%Wind =                      &   
               INT(SUM(Buffer(1:HoursPerTS))/HoursPerTS)             ! Average wndspd for ts
            Buffer(1) = HourMtr(H,I)%Dir                             ! Copy windspeed
            HourMtr(H:H+HoursPerTS-1,I)%Dir =Buffer(1)               ! Wind dir at start of ts 
         ENDDO                                                       ! End loop over hours 
      ENDDO                                                          ! End loop over zones  
   ENDIF                                                             ! End timestep logic                                        
   DO I=1, NumZones                                                  ! Loop over zones 
      DO H=1,NumHours                                                ! Loop over hours in sim
         N = "TEMPCAT"                                               ! Set TEMPCAT distribution name 
         IF (FuncNum(N,NO)>0)  THEN                                  ! If TEMPCAT is defined 
            HV => HourMtr(H:H,I)%Temp                                ! Pointer to temperature   
            CALL EvalFunc(Name=N,I1=HV,IRes=HourMtr(H:H,I)%TempCat)  ! Get TempCat for current hour                
         ENDIF                                                       ! End of TempCat logic block
         N = "HUMIDCAT"                                              ! Set HUMIDCAT distribution name 
         IF (FuncNum(N,NO)>0)  THEN                                  ! If HUMIDCAT is defined 
            HV => HourMtr(H:H,I)%Humid                               ! Pointer to humidity   
            CALL EvalFunc(Name=N,I1=HV,IRes=HourMtr(H:H,I)%HumidCat) ! Get HumidCat for current hour                
         ENDIF                                                       ! End of HumidCat logic block
         N = "PRECIPCAT"                                             ! Set PRECIPCAT distribution name 
         IF (FuncNum(N,NO)>0)  THEN                                  ! If PRECIPCAT is defined 
            PR => HourMtr(H:H,I)%Precip                              ! Pointer to precipitation   
            CALL EvalFunc(Name=N,C1=PR,IRes=HourMtr(H:H,I)%PrecipCat)! Get PrecipCat for current hour                
         ENDIF                                                       ! End of PrecipCat logic block
         N = "WINDCAT"                                               ! Set WINDCAT distribution name 
         IF (FuncNum(N,NO)>0)  THEN                                  ! If WINDCAT is defined 
            HV => HourMtr(H:H,I)%Wind                                ! Pointer to windspeed   
            CALL EvalFunc(Name=N,I1=HV,IRes=HourMtr(H:H,I)%WindCat)  ! Get WindCat for current hour                
         ENDIF                                                       ! End of WindCat logic block
         N = "DIRCAT"                                                ! Set DIRCAT distribution name 
         IF (FuncNum(N,NO)>0)  THEN                                  ! If DIRCAT is defined 
            HV => HourMtr(H:H,I)%Dir                                 ! Pointer to wind dir   
            CALL EvalFunc(Name=N,I1=HV,IRes=HourMtr(H:H,I)%DirCat)   ! Get DirCat for current hour                
         ENDIF                                                       ! End of DirCat logic block
      ENDDO                                                          ! End loop over hours
      DO J=1, NumDays                                                ! Loop over days  
         N = "MAXTEMPCAT"                                            ! Set first distribution name
         IF (FuncNum(N,NO)>0) THEN                                   ! If MaxTempCat distrib is defined
            MT => TemData(J:J,I)%MaxTemp                             ! Pointer to max temp value
            CALL EvalFunc(Name=N,I1=MT,IRes=TemData(J:J,I)%TempCatM) ! Get TempCatM for current day
         ENDIF                                                       ! End of MaxTempCat logic block
         N = "AVGTEMPCAT"                                            ! Set first distribution name
         IF (FuncNum(N,NO)>0) THEN                                   ! If AvgTempCat distrib is defined
            AT => TemData(J:J,I)%AvgTemp                             ! Pointer to mean temp value
            CALL EvalFunc(Name=N,I1=AT,IRes=TemData(J:J,I)%TempCatA) ! Get TempCatA for current day
         ENDIF                                                       ! End of AvgTempCat logic block
      ENDDO                                                          ! End loop over days
   ENDDO                                                             ! End loop over zones
   WD => Calen%DayofWeek                                             ! Pointer to day of week vector
   N = "DIARYPOOLS"                                                  ! Name of the pools distribution
   I = FuncNum(N,YES)                                                ! Stop if distrib is undefined
   DO I=1,NumZones                                                   ! Loop over temperature zones
      NMiss = COUNT(HourMtr(:,I)%Temp==-999)                         ! Count the unassigned TempCats
      TotMiss = TotMiss+NMiss                                        ! Add site total to overall total 
      IF (NMiss>0) WRITE(LogU,*) "Zone ", ZList(I)%Name," missing", &
         Nmiss, "hourly meteorology records"                         ! Write count of missing to log
      MT  => TemData(:,I)%MaxTemp                                    ! Pointer to MaxTemp vector
      AT  => TemData(:,I)%AvgTemp                                    ! Pointer to AvgTemp vector
      CALL EvalFunc(Name=N,I1=MT,I2=AT,I3=WD,IRes=TemData(:,I)%Pool) ! Get pool numbers from distrib
   ENDDO                                                             ! Continue with next zone
   IF (TotMiss>0) CALL Fatal(4)                                      ! Stop if any temp data is missing
   IF (TimeStepsPerDay<24) DEALLOCATE(Buffer)                        ! Deallocating internal array
   IF (DebugLevel>0) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth 
END SUBROUTINE ReadMeteorologyData                                   ! Return to main APEX program 

SUBROUTINE ReadAirQuality                                            ! Called from main APEX program
   CHARACTER (LEN=LEN5) :: String, String1, UpString, SubS           ! Buffers for input records
   CHARACTER(LEN=LEN2)  :: N                                         ! Buffer for dist and site names   
   CHARACTER (LEN=1)    :: Char1                                     ! Buffer for a single character
   INTEGER (KIND=IK)    :: Date=0, I=0, J, PL, TS, LType             ! Local integer variables
   INTEGER (KIND=IK)    :: NMiss=0, TotMiss=0, H                     ! Local integer variables
   REAL (KIND=RK), POINTER :: HV(:)                                  ! Pointers for distrib arguments   
   REAL (KIND=RK), ALLOCATABLE :: Concs(:)                           ! Buffer for air concs
   ! Global constants: AQU, HOURSDAY, IK, LEN2, LEN3, LV_FMT, LV_MSG, LogU, ST_FMT, ST_MSG, YES 
   ! Global vars used: DebugLevel, DSTAdjust, NumDistricts, NumHours, RBBackgnd, RBFactor 
   ! Global vars set : AQData, Ios, ProcLevel, ProcName
   ! Module vars used: EndJulian, StartJulian
   ! Intrinsic procs : ADJUSTL, ANY, CLOSE, COUNT, INDEX, MINVAL, READ, SUM, TRIM, WRITE
   ! APEX procedures : DaylightSaving, Fatal, GetLine, OpenInput, Up 
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "ReadAirQuality"                            ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging  
   ALLOCATE(Concs(1:TimeStepsPerDay),STAT=AllocErr)                  ! Allocate concentration buffer  
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(AQCond(NumPollutants,(-1*TimestepsPerDay+1):NumTS,-1:NumDistricts),STAT=AllocErr) ! Make space for TS AQ cond data
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc    
   IF (ModelAQVar == YES) THEN                                       ! Expect to read AQ distribs       
      ALLOCATE (AQDistr(NumPollutants,NumDistricts,                 &
         NumDays,TimeStepsPerDay),STAT=AllocErr)                     ! Space for AQ distributions
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc
      DO PL=1, NumPollutants                                         ! Loop over pollutants
         I = 0                                                       ! Initialize district counter                  
         J = 0                                                       ! Initialize day counter
         CALL OpenInput(AQU,name=Pol(PL)%AQName,h=no)                ! Open air quality file for input
         DO                                                          ! Loop over recs in input file
            CALL GetLine(AQU, String1, LType)                        ! Get one record from input file
            IF (Ios(AQU)<0) EXIT                                     ! Exit loop on end-of-file
            IF (INDEX(Up(String1),"NAME")==1) THEN                   ! If keyword is NAME, process it
               SubS = TRIM(ADJUSTL(String1(LType+1:)))               ! Get part on right of '='
               IF (SubS=="") CALL Fatal(1)                           ! If site has no name, then error
               I = MINVAL(DList%Index,MASK=(DList%Name==SubS))       ! Find name in Dlist names
            ENDIF                                                    ! I is the district number 
            IF (I<=0) CYCLE                                          ! Skip line if district not valid
            IF (I>NumDistricts) CYCLE                                ! This means SubS is not on DList
            IF (LType>=0) CYCLE                                      ! Only numeric data below here
            UpString=Up(String1)                                     ! Make uppercase version of line
            READ(UpString,*,IOSTAT=IOS(AQU)) Date, TS, Char1         ! Read date, timestep, shape      
            IF (Date<10000000) CALL Fatal(7)                         ! Wrong format data
            J = 1 + JulianDay(Date) - StartJulian                    ! Day number for this record
            IF (J<1.OR.J>NumDays) CYCLE                              ! Drop dates out of range
            UpString = UpString(INDEX(UpString,Char1):)              ! Drop date and hour from 
            AQDistr(PL,I,J,TS) = ReadDist(UpString)                  ! Read distribution parameters 
         ENDDO                                                       ! End loop over records
         DO I=1,NumDistricts                                         ! Loop over air quality districts
            IF (ANY(AQDistr(PL,I,:,:)%Shape==0)) THEN                ! Examine distribution shapes    
               WRITE(LogU,*) "District ", DList(I)%Name,            &
                  " missing AQ distribution data"                    ! Write message to log
               CALL Fatal(1)                                         ! Mising AQ distribution data 
            ENDIF                                                    ! End error logic
         ENDDO                                                       ! End loop over districts     
      ENDDO                                                          ! End loop over pollutants
   ELSE                                                              ! Otherwise expect raw AQ data    
      AQData = -1.                                                   ! Initialize data to negative
      DO PL=1, NumPollutants                                         ! Loop over pollutants
         I = 0                                                       ! Initialize district counter                  
         J = 0                                                       ! Initialize day counter
         CALL OpenInput(AQU,name=Pol(PL)%AQName,h=no)                ! Open air quality file for input
         DO                                                          ! Loop over input records
            CALL GetLineLong(AQU,String,LType)                       ! Get one record from input file
            IF (Ios(AQU)/=0) EXIT                                    ! If end-of-file, exit DO loop
            IF (INDEX(UpLong(String),"NAME")==1) THEN                ! If keyword is NAME, process it
               SubS = TRIM(ADJUSTL(String(LType+1:)))                ! Get part on right of '='
               IF (SubS=="") CALL Fatal(2)                           ! If site has no name, then error
               I = MINVAL(DList%Index,MASK=(DList%Name==SubS))       ! Find name in Dlist names
            ENDIF                                                    ! I is the district number 
            IF (I<=0) CYCLE                                          ! Skip line if district not valid
            IF (I>NumDistricts) CYCLE                                ! This means SubS is not on DList
            IF (LType>=0) CYCLE                                      ! Only numeric data below here
            READ (String,*,IOSTAT=Ios(AQU)) Concs, Date              ! Get concs and YYYYMMDD          
            IF (Date==0) CALL Fatal(6)                               ! Wrong type of data
            IF (Ios(AQU)/=0) CALL Fatal(3)                           ! Data not numeric
            J = 1 + JulianDay(Date) - StartJulian                    ! Day number for this record
            IF (J<1.OR.J>NumDays) CYCLE                              ! Drop dates out of range
            AQData(PL,TimeStepsPerDay*(J-1)+1:TimeStepsPerDay*J,I)  &
               = Concs                                               ! Copy all values to AQData      
         ENDDO                                                       ! Continue loop over input records
         CLOSE(AQU)                                                  ! Close the air quality file
         DO I=1,NumDistricts                                         ! Loop over air quality districts
            NMiss = COUNT(AQData(PL,1:,I)<0)                         ! Count the negative concs
            IF (POL(PL)%UseRoadAQ==NO .AND. INDEX(Up(DList(I)%Name),"ROAD")>0)  & ! If the missing are...
                NMiss = 0                                            ! ... roadway and not needed
            TotMiss = TotMiss+NMiss                                  ! Add site total to overall total
            IF (NMiss>0) WRITE(LogU,*) "District ", DList(I)%Name,  &
               " missing", Nmiss, " air concentrations"              ! Write count of missing to log
         ENDDO                                                       ! Continue with next zone
         IF (TotMiss>0) CALL Fatal(4)                                ! Stop if any air data is missing 
         IF (DSTAdjust==YES) CALL DaylightSaving(PL)                ! Adjust for Daylight Saving Time
         AQData(PL,1-TimeStepsPerDay:0,:) =                         &
            AQData(PL,1:HOURSDAY,:)                                  ! Repeat first day of AQ data
         DO I=-1*TimestepsPerDay+1,NumTS                             ! Loop over all timesteps
            AQData(PL,I,-1) = LeaverMult*                           &
               (SUM(AQData(PL,I,1:))/NumDistricts)+LeaverAdd         ! AQ for leavers is city average      
         ENDDO                                                       ! End loop over timesteps
         IF (ANY(AQData(PL,:,-1)<0)) CALL Fatal(5)                   ! Stop if any AQ out < 0
         DO I=0,NumDistricts                                         ! Loop over columns of array 
            WHERE(AQData(PL,:,I)>RBBkgnd) AQData(PL,:,I) =          &
               RBFactor *(AQData(PL,:,I)-RBBkgnd)+RBBkgnd            ! Adjust AQ data for rollback
         ENDDO                                                       ! Next column of array  
      ENDDO                                                          ! Finished with pollutant
   ENDIF                                                             ! End AQ type logic
   DO PL=1,NumPollutants                                             ! Loop over pollutants
       DO I=-1, NumDistricts                                         ! Loop over districts   
          DO H=1,NumTS                                               ! Loop over TSteps in sim
                 N = "AQCONDITIONAL1"                                ! Set AQCOND1 distribution name 
                 IF (FuncNum(N,NO)>0)  THEN                          ! If AQCOND1 is defined 
                    HV => AQData(PL,H:H,I)                           ! Pointer to AQ                         
                    CALL EvalFunc(Name=N,R1=HV,IRes=AQCond(PL,H:H,I)%AQCond1)! Get AQCOND1 for current hour                           
                 ENDIF                                               ! End of TempCat logic block
                 N = "AQCONDITIONAL2"                                ! Set AQCOND2 distribution name 
                 IF (FuncNum(N,NO)>0)  THEN                          ! If AQCOND2 is defined 
                    HV => AQData(PL,H:H,I)                           ! Pointer to AQ   
                    CALL EvalFunc(Name=N,R1=HV,IRes=AQCond(PL,H:H,I)%AQCond2)! Get AQCond2 for current hour                             
                 ENDIF                                               ! End of HumidCat logic block
                 N = "AQCONDITIONAL3"                                ! Set AQCOND3 distribution name 
                 IF (FuncNum(N,NO)>0)  THEN                          ! If AQCOND3 is defined 
                    HV => AQData(PL,H:H,I)                           ! Pointer to AQ                        
                    CALL EvalFunc(Name=N,R1=HV,IRes=AQCond(PL,H:H,I)%AQCond3)! Get PrecipCat for current hour                                                 
                 ENDIF                                               ! End of AQCond3 logic block
                 N = "AQCONDITIONAL4"                                ! Set AQCOND4 distribution name 
                 IF (FuncNum(N,NO)>0)  THEN                          ! If AQCOND4 is defined 
                    HV => AQData(PL,H:H,I)                           ! Pointer to AQ                       
                    CALL EvalFunc(Name=N,R1=HV,IRes=AQCond(PL,H:H,I)%AQCond4)! Get AQCond4 for current hour                                               
                 ENDIF                                               ! End of WindCat logic block
                 N = "AQCONDITIONAL5"                                ! Set AQCOND5 distribution name 
                 IF (FuncNum(N,NO)>0)  THEN                          ! If AQCOND5 is defined 
                    HV => AQData(PL,H:H,I)                           ! Pointer to AQ                          
                    CALL EvalFunc(Name=N,R1=HV,IRes=AQCond(PL,H:H,I)%AQCond5)! Get AQCond5 for current hour                                                     
                 ENDIF                                               ! End of DirCat logic block
          ENDDO                                                      ! End loop over TSteps
       ENDDO                                                         ! End loop over zones
      AQCond(PL,1-TimeStepsPerDay:0,:) = AQCond(PL,1:TimeStepsPerDay,:)! Repeat first day of AQ data      
   ENDDO                                                             ! End loop over pollutant   
   DEALLOCATE(Concs)                                                 ! Deallocate internal array
   IF (DebugLevel>0) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE ReadAirQuality                                        ! Return to main APEX program

SUBROUTINE GenerateAirQuality(P,PL)                                  ! Called from main APEX program
   INTEGER (KIND=IK),INTENT(IN) :: P, PL                             ! Input arguments (profile,pollut)
   INTEGER (KIND=IK)   :: I=0                                        ! Local integer variables
   INTEGER (KIND=IK)   :: Day, TS, Step                              ! Local integer variables
   REAL    (KIND=RK)   :: Samp(1)                                    ! Buffers for hourly air concs
   ! Global constants: AQU, HOURSDAY, IK, LEN2, LEN3, LV_FMT, LV_MSG, LogU, ST_FMT, ST_MSG, YES 
   ! Global vars used: DebugLevel, DSTAdjust, NumDistricts, NumHours, RBBackgnd, RBFactor 
   ! Global vars set : AQData, Ios, ProcLevel, ProcName
   ! Module vars used: EndJulian, StartJulian
   ! Intrinsic procs : ADJUSTL, ANY, CLOSE, COUNT, INDEX, MINVAL, READ, SUM, TRIM, WRITE
   ! APEX procedures : DaylightSaving, Fatal, GetLine, OpenInput, Up 
    IF (ModelAQVar==NO)  RETURN;                                     ! Return if using raw AQ data
    ProcLevel = ProcLevel+1                                          ! Increment procedure depth
    ProcName(ProcLevel) = "GenerateAirQuality"                       ! Procedure name for messages
    IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel) ! Message for debugging  
    AQData = -1.                                                     ! Initialize data to negative
    DO I=1,NumDistricts                                              ! Loop over districts
       DO Day=1,NumDays                                              ! Loop over days 
          DO TS=1,TimeStepsPerDay                                    ! Loop over timesteps       
             Step=(Day-1)*TimeStepsPerDay+TS                         ! Overall hour number
             CALL SampleDist(AQDistr(PL,I,Day,TS),Samp)              ! Sample from timestep dist
             AQData(PL,Step,I)=Samp(1)                               ! Set hourly AQ Value              
          ENDDO                                                      ! End hours
       ENDDO                                                         ! End days
    ENDDO                                                            ! End districts
    IF (DSTAdjust==YES) CALL DaylightSaving(PL)                     ! Adjust for Daylight Saving Time
    AQData(PL,1-TimeStepsPerDay:0,:) = AQData(PL,1:TimeStepsPerDay,:)! Repeat first day of AQ data
    AQData(PL,:,-1) = LeaverMult*                                    &
       (SUM(AQData(PL,:,1:),DIM=2)/NumDistricts)+LeaverAdd           ! AQ for leavers is city average
    IF (ANY(AQData(PL,:,-1)<0)) CALL Fatal(1)                        ! Stop if any AQ out < 0
    DO I=0,NumDistricts                                              ! Loop over columns of array 
       WHERE(AQData(PL,:,I)>RBBkgnd) AQData(PL,:,I) = RBFactor *     & 
          (AQData(PL,:,I)-RBBkgnd)+RBBkgnd                           ! Adjust AQ data for rollback
    ENDDO                                                            ! Next column of array  
    IF (DebugLevel>0) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel) ! Message for debugging
    ProcLevel = ProcLevel-1                                          ! Decrement procedure depth
END SUBROUTINE GenerateAirQuality                                    ! Return to main APEX program

SUBROUTINE DaylightSaving(PL)                                        ! Called from ReadAirQuality
   INTEGER (KIND=IK), INTENT(IN)  :: PL                              ! Pollutant #
   INTEGER (KIND=IK) :: Year,Z,Start,Leap,JulOn,JulOff,On,Off,I      ! Local integer variables
   INTEGER (KIND=IK) :: OnTS,OffTS,K,J,DurTS                         ! Local integer variables
   ! Global constants: HOURSDAY, IK, LogU, LV_MS2, SL_FM2, ST_MS2
   ! Global vars used: DebugLevel, EndDate, NumDays, StartDate
   ! Global vars set : AQData
   ! Module vars used: EndJulian, StartJulian
   ! Intrinsic procs : EOSHIFT, MOD, WRITE
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "DaylightSaving"                            ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,SL_FM2) ST_MS2, ProcName(ProcLevel)  ! Message for debugging   
   DO Year = StartDate/10000, EndDate/10000                          ! Loop over years in simulation
      Z     = Year-1                                                 ! Base is previous year
      Start = (Z*365)+(Z/4)-(Z/100)+(Z/400)                          ! End Julian day for base year
      Leap  = 0                                                      ! Default is non-leap year
      IF (MOD(Year,4)==0) Leap = 1                                   ! Normal leap year condition 
      IF ((MOD(Year,100)==0).AND.(MOD(Year,400)/=0)) Leap = 0        ! Century bounds condition
      JulOn  = Start+73+Leap-MOD(Start+Leap+3,7)                     ! First DST Julian day 
      IF (Year<1986) JulOn = Start+120+Leap-MOD(Start+Leap+1,7)      ! First DST Julian day (pre-1986)
      IF (Year<2007) JulOn = Start+97 +Leap-MOD(Start+Leap+6,7)      ! First DST Julian day (pre-2007)
      JulOff = Start+311+Leap-MOD(Start+Leap+3,7)                    ! Last DST day      
      If (Year<2007) JulOff = Start+304+Leap-MOD(Start+Leap+3,7)     ! Last DST day (pre-2007)
      IF (StartJulian > JulOff) CYCLE                                ! No DST for this year
      IF (EndJulian   < JulOn)  EXIT                                 ! No more DST for simulation
      On = HOURSDAY*(JulOn-StartJulian)+2                            ! First DST hour this year
      IF (On < 0) On = 2                                             ! If simulation starts in summer
      OnTS = On*TIMESTEPSPERHOUR                                     ! Convert On hour to TS
      Off = HOURSDAY*(JulOff-StartJulian)+2                          ! Last DST hour this year
      IF (Off > HOURSDAY*NumDays) Off = HOURSDAY*NumDays             ! If simulation ends in summer
      OffTS = Off*TIMESTEPSPERHOUR                                   ! Convert Off hour to TS
      DurTS = OffTS-OnTS
      DO I=0,NumDistricts                                            ! Loop over all districts
         DO K=0,DurTS-1                                              ! Loop over all DST time steps
            AQData(PL,OffTS-K,I) = AQData(PL,OffTS-TIMESTEPSPERHOUR-K,I) ! Shift AQ Data
         ENDDO                                                       ! End loop over timesteps
         AQData(PL,OnTS:OnTS+TIMESTEPSPERHOUR,I) =                  &
            AQData(PL,OnTS-TIMESTEPSPERHOUR+1:OnTS,I)                ! Duplicate previous hour for extra
      ENDDO                                                          ! Next column of array 
   ENDDO                                                             ! Continue with next year
   WRITE (LogU,*) "   Completed Daylight Savings Time adjustment"    ! Put message on log file
   IF (DebugLevel>0) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE DaylightSaving                                        ! Return to ReadAirQuality

END MODULE SpaceTimeModule                                           ! End of SpaceTimeModule code
